"""
Author: Daniel Tang, Agriculex Inc.
Date: Feb 28, 2021
Purpose: use Python language to create a interface for ESC-2 seed counter
----------------------------------------------------------------------------------------------
|June 4,2021 | move to raspberry pi 4  using Visual studio Code| python version:3.7.3
----------------------------------------------------------------------------------------------
|May 4, 2022 | finish total version 1.0
|-------------------------------------------------------------------------------------------
| Dec 12, 2023 | for new circuit board version 1.0
-------------------------------------------------------------------
| May 7, 2024 | modified dump process and ensure next batch not to wait when envelope has been opened
|----------------------------------------------------------------------------------------------------
|June 19,2024 | add disable button when dumping and change dumping to cleanout
|-----------------------------------------------------------------------------------------------------
|August 9, 2024 | add Trunoff19() method to force turnoff main circuit board
|------------------------------------------------------------------------------------------
|Nov 25, 2024 | 1.add a input speed option in batch select menu. 2. add a thread to monitor sensitvity if it gose to higher
|----------------------------------------------------------------------------------------------------------
|Dec 12, 2024 |  Add command 'Stop', 'Start' at batch count mode
|----------------------------------------------------------------------------------------------
|Dec 24, 2024 | carousel envelope present model:a pulse signal from high level to low level and last 0.75 second then back to 
|             | high level 
|----------------------------------------------------------------------------------------------------------------------------
|Jan 09, 2025 | add 0.3 second time delay for edge down and up of envelope presnet
|------------------------------------------------------------------------------------------
|Jan 31, 2025 | prevent first packet skip
|--------------------------------------------------------------------------------------
"""

from concurrent.futures import thread
from fileinput import filename
import os,stat
from readline import set_completer_delims
import tkinter as TK
from tkinter.constants import CENTER, COMMAND, END, LEFT, NO, RIGHT, TRUE, Y
import tkinter.ttk as TTK
from tkinter.font import Font
import threading 
from datetime import datetime
import time
import logging

#from matplotlib.pyplot import title
from rs485_server import ModbusServerRS
#from typing import Callable, Text
from openpyxl import Workbook,load_workbook
from openpyxl.utils import get_column_letter
import cropPara as crp
from esc2Kernel import cValve,cTurntab,cCount_N,cEnvenlop, cPower,rs485set
from batch_inf import batchInf 
from esc2_dialogue import File_dialogue
from esc2_message import Message_dialogue

#from pynput.keyboard import Key,Controller

page_Back=''   #claim a globle varible
rs232_carousel='1' #0--no carousel mode,1--carousel mode

itemTv_id=[] #treeview  list
count_mode=("idle","totalize","Batch","load_F","Dump")

#initial controlmodule
valve=cValve()  #initial valve class
turnTable=cTurntab()  #initial turntab class
count=cCount_N()      #initial count
envelope=cEnvenlop()    #initial envelope class
pi_shutdown=cPower()
rs485Set=rs485set()


#initial batch data inf
obj_batchInf=batchInf()

#initial thread event object
eventThread=threading.Event()
eventThread.set()  #set flag to true, no block
threadExit_Flag=True #counter thread flag

#initial logging 
format1="%(asctime)s: %(message)s"
logging.basicConfig(filename='/home/pi/esc2/log_file/esc2log.log',
                    filemode='a',
                    format=format1,
                    level = logging.INFO,
                    datefmt="%H:%M:%S")

class frlsWin():
    
    #this is a  base class 
    def __init__(self):
        self.bPoweroff_flag=False  #False--power on, True--Power off    
        self.countType=""                             
        self.root = TK.Tk()
        #set window size
        self.root.geometry("1280x720+0+0")
        self.root['bg']='gray'
        #set label
        self.myFont=Font(family="Times New Roman",size=20)
        
        self.label = TK.Label(self.root,text="Agriculex ESC2 Seed Counter  verison 1.0",
                              bg='gray',
                              font=self.myFont)
        self.label.pack(pady=10,ipadx=5)

        #create a frame as line
        self.frame =TK.Frame(self.root,bg="blue",bd=10,relief='ridge').pack(fill=TK.BOTH)
        #create a frameless window
        self.root.maxsize(1280,720) #pixel 1280x720 for 9" touch panel
        self.root.attributes("-fullscreen","true")
    
        #create a thread to monitor power
        self.powerThread=threading.Thread(target=self.powerHandler_c, 
                                    name='powerHandler',args=('Wating for close...',)
                                     )                   
        self.powerThread.setDaemon(True)
        self.powerThread.start()
    
      
    def powerHandler_c(self,strVal):
        pi_shutdown.power_detect()
        while True:
            time.sleep(0.1)
            if pi_shutdown.isPowerOn():    
                logging.info('Power shut off')
                self.bPoweroff_flag=True
                if self.countType =="":
                    self.lblWarning=TK.Label(self.root,text='System is shutdown...',
                                            bg='gray',fg='blue',
                                            relief=TK.RAISED,
                                            font=self.myFont)
                    self.lblWarning.place(x=500,y=250)
                    time.sleep(2)
                    pi_shutdown.TurnOff19()
                    os.system('sudo shutdown -h now')
                else:
                    self.bPoweroff_flag=True
                
                break
                
            if page_Back=='back':
                pi_shutdown.cleanDetect()
                break
                    
'''
    define a digit keyboard
    start: July 7,2021
''' 
class digit_keyboard():
    def __init__(self,parent,speedVal):
        self.root=parent
        self.speedText=speedVal

        #draw digit key board
        self.digi_B=TK.Toplevel(self.root)
        self.digi_B.title("Digit Key")
        self.digi_B.geometry("400x305+200+200")
   
        self.digi_B.resizable(False,False)
        self.digi_B.overrideredirect(True)
        self.digi_B.grab_set()

        self.digikey_board()
        self.enSp.select_range(0,'end')
        self.enSp.focus()

        self.digi_B.wait_window()

    def digikey_board(self):#create a numpad
       
        #create frames
        lblFrame=TK.Frame(self.digi_B,bg='darkslategray')
        lblFrame.pack(fill=TK.X,pady=5,padx=5)
        keyFrame=TK.Frame(self.digi_B,bg='darkslategray')
        keyFrame.pack(fill=TK.BOTH,pady=5,padx=8)
        
        #create entry interface
        self.speedVar=TK.StringVar()
        self.speedVar.set(self.speedText)
        lb1=TK.Label(lblFrame,text='Speed:',bg='darkslategray',font=('Courier',20))
        lb1.pack(side=TK.LEFT,padx=5)
        self.enSp=TK.Entry(lblFrame,textvariable=self.speedVar,
                        font=('Courier',24),
                        width=6
                        )
        self.enSp.pack(side=TK.LEFT,padx=40)
        lb2=TK.Label(lblFrame,text=self.speedText,font=('Courier',18)) 
        lb2.pack(side=TK.RIGHT,padx=5)  

        #create key pad
        keyD = ['7','8','9',' 0 ',
                    '4','5','6','<--',
                '1','2','3','Ok '
                ] 
        
        r=0
        c=0
        for label in keyD:
            cmd=lambda x=label:self.click_d(x)
            key=TK.Button(keyFrame,text=label,bg ='darkgrey',
                        font=('Courier',30),relief='raised',
                        command=cmd
                        )
            key.grid(row=r,column=c,padx=1,pady=1,ipadx=15,ipady=8,stick=TK.W)
        
            c +=1
            if c==4:
                c=0
                r +=1
        
    def click_d(self,str_):
        #special for speed pad
        if str_=='<--':
            currentT=self.speedVar.get()
            self.speedVar.set(currentT[:-1])
            self.enSp.icursor('end')

        elif str_=='Ok ':
            str1=self.speedVar.get()
            if str1=="":
                    self.digi_B.destroy()   #no change

            elif int(str1)>200 :
                self.digi_B.destroy()
                Message_dialogue(self.root,'Maxumum is 200 !','ok','Reminder')             
                self.speedVar.set('200')
                self.speedText=self.speedVar.get()     
                
            else:   
                self.speedText=str1
                self.digi_B.destroy()

        else:
            currentT=self.speedVar.get()
            if str_==' 0 ':
                str_='0'
                if currentT=="" :
                    self.digi_B.destroy()
                    Message_dialogue(self.root,'First number must greater than 0','ok','Reminder !')
                    self.speedVar.set('50')
                    self.speedText=self.speedVar.get()   
                else:
                    self.speedVar.set(currentT+str_)
                    self.enSp.icursor('end') 
            else:
                if self.enSp.select_present():
                    self.speedVar.set(str_)
                    self.enSp.select_clear()
                    self.enSp.icursor('end')
                else:
                    self.speedVar.set(currentT+str_)
                    self.enSp.icursor('end')

'''
    Define a keyboard
    start: oct 29,2021
''' 
class keyF_keyboard():
    def __init__(self,parent,position,labelName,d_path):
    
        self.root=parent
        self.fileName=''
        self.path=d_path
        self.lab=labelName
        self.bExit_flag=False #True-button pressed

        self.iFlag=1 #0--lower case,1--upper case
        geStr='880x465+'+str(position[0])+'+'+str(position[1])
       
        #draw digit key board
        self.keyF_B=TK.Toplevel(self.root)
        self.keyF_B.title("Keyboard")

        self.keyF_B.geometry(geStr)
        self.keyF_B.attributes('-topmost',True)
        self.keyF_B.resizable(False,False)
        self.keyF_B.overrideredirect(True)
       
        topFrame=TK.Frame(self.keyF_B,bg='light slate blue')
        topFrame.pack(fill='x',ipady=10)
        fileNameLbl=TK.Label(topFrame,text=labelName,bg='gray',font=('Courier',24))
        fileNameLbl.pack(side='left',padx=20,ipadx=10,ipady=10)
        self.enVar=TK.StringVar()
        if labelName=='File name:':
            self.enVar.set('Esc2-1')
            self.nameEntry=TK.Entry(topFrame,textvariable=self.enVar,
                                    font=('Courier',24),
                                    width=16,bg='gray')
        else:
            self.enVar.set('')
            self.nameEntry=TK.Entry(topFrame,textvariable=self.enVar,
                                    font=('Courier',24),
                                    width=28,bg='gray')
        self.nameEntry.pack(side='left',ipady=5)
        if labelName=='File name:':
            foldBtn=TK.Button(topFrame,text='Directory',
                              font=('Courier',24),
                              bg='gray',command=self.click_fold)
            foldBtn.pack(side='right',ipadx=5,padx=50)
        self.nameEntry.select_range(0,'end')
        self.nameEntry.focus()
    
        self.keyF_board()

        self.keyF_B.wait_window()

    def keyF_board(self):#create a numpad
        #Create main frame
        mainFrame=TK.Frame(self.keyF_B,bg='light slate blue')
        keyFrame=[]
        #create frames
        i=0
        for i in range(5):
            #p=TK.Frame(mainFrame,bg='darkslategray')
            p=TK.Frame(mainFrame,bg='light slate blue')
            keyFrame.append(p)
        #create key pad
        keyF = [['1','2','3','4','5','6','7','8','9','0','<--'],
                ['Cap','Q','W','E','R','T','Y','U','I','O','P'],
                ['A','S','D','F','G','H','J','K','L','_',';'],
                ['Z','X','C','V','B','N','M',',','Enter'],
                ['Exit']
                ] 
        c=1
        self.key=[['' for col in range(11)]for row in range(5)]
        x_space=2
        for row_1 in range(5):
            for label in keyF[row_1]:
                cmd=lambda x=label:self.click_d(x)
                keyBtn=TK.Button(keyFrame[row_1],text=label,bg ='darkgrey',
                                font=('Courier',36),relief='raised',
                                command=cmd
                            )
                self.key[row_1][c-1]=keyBtn
                if row_1==0 or row_1==1:
                    keyBtn.grid(row=row_1+1,column=c,padx=1,pady=1,ipadx=6,ipady=1)
                    x_space=20
                elif row_1==2:
                    keyBtn.grid(row=row_1+1,column=c,padx=1,pady=1,ipadx=6,ipady=1)
                    x_space=45
                elif row_1==3:
                    keyBtn.grid(row=row_1+1,column=c,padx=1,pady=1,ipadx=6,ipady=1)
                    x_space=60
                else:
                    keyBtn.pack(ipadx=10)
                    x_space=120
                c +=1
                
            keyFrame[row_1].pack(fill=TK.X,padx=x_space)
            c=1

        mainFrame.pack(fill='x',pady=10)

    def click_d(self,str_):
        #special for speed pad
        cur_T=self.enVar.get()
        if str_=='<--':
            self.enVar.set(cur_T[:-1])
            self.nameEntry.icursor('end')

        elif str_=='Enter':
            #set default fold
            #self.fileName='/home/pi/esc2/dataSave/'+self.enVar.get()+'.xlsx'
            if self.lab=='Barcode:':
                self.fileName=self.enVar.get()
            else:
                self.filename=self.path
            self.bExit_flag=True
            self.keyF_B.destroy()
        elif str_=='Cap':
            for row in range(1,4,1):
                for col in range(11):
                    Btn=self.key[row][col]
                    if Btn !='':
                        c=Btn['text']
                        if c !='Cap':
                            if c !='' and c !='_' and c !='Enter':
                                if self.iFlag==1: #uppcase
                                    Btn['text']=c.lower()
                                else:
                                    Btn['text']=c.upper()
            if self.iFlag==1:
                self.iFlag=0
            else:
                self.iFlag=1
        elif str_=='Exit':
            self.keyF_B.destroy()
            self.bExit_flag=False
        else:
            self.nameEntry.focus()
            if self.nameEntry.select_present():
                self.enVar.set(str_)
                self.nameEntry.select_clear()
                self.nameEntry.icursor('end')
            else:
                if self.iFlag==0:
                    self.enVar.set(cur_T+str_.lower())
                else:
                    self.enVar.set(cur_T+str_.upper())
                self.nameEntry.icursor('end')
            
    def click_fold(self):
        self.keyF_B.destroy()
        fMsg=File_dialogue(self.root,self.enVar.get())
        if fMsg.iflag=='OK':
            self.fileName=fMsg.dir_backVari.get()
            self.bExit_flag=True
        else:
            self.bExit_flag=False
        '''
        Pname=Message_dialogue(self.root,'Save Sample ?','yes-no','Reminder !')
      
        if Pname.iflag=='Cancel' :#cancel button
             self.fileName=self.enVar.get()
             self.bExit_flag=True
        else:
             self.fileName=Pname
        '''
    def get_Filename(self):
        return self.fileName
"""
   define a framless base class
"""
"""
define a pop up base class 
"""
class popupWin():
    def __init__(self,parent,title,object0,speed,*args,**kwargs):# args is a tuple of(width,height,x,y,bg,bd)
        self.root=parent
        #add define variabls for entry
        self.batchinf_b=object0

        self.tt=[]  #frame dimension
        self.tt=list(*args)
        self.topWin=TK.Toplevel(parent,bg='gray',bd=20)
        self.topWin.title(title) 
        self.topWin.transient(self.root)
        self.btn_flag=0 # 0-cancel, 1--ok button

        self.batchNo = TK.StringVar()
        self.batchNo.set("1") #default set 1 batch
        self.noperBatch =TK.StringVar()
        self.noperBatch.set("1") #default set 10
        self.noperPacket = TK.StringVar()
        self.noperPacket.set("100") #default set 50 kernel for one packet
        self.speed = TK.StringVar()
        self.speed.set(speed)
        
        #define font of characters
        self.myFont=Font(family="Times Newn Roman",size=24)
        #Window size and position
        self.rootx=0
        self.rooty=0

        self.par=self.winPo()
        self.topWin['bg']='cadet blue'
        self.topWin.geometry(self.par)
    
        #Ceate a frame for a position of Ok and cancel buttom
        self.okcancelFrm=TK.Frame(self.topWin,bg='cadet blue')
        
        #define ok button
        self.okBtn=TK.Button(self.okcancelFrm,text="OK",
                             font=self.myFont,
                             command=self.okBtn_click,bg='gray').pack(side=TK.LEFT,pady=2,padx=30,ipadx=10,ipady=10)
        
        self.cancelBtn=TK.Button(self.okcancelFrm,text="Cancel",
                                 font=self.myFont,
                                 command=self.cancelBtn_click,bg='gray').pack(side=TK.LEFT,padx=30,ipadx=10,ipady=10)
        self.okcancelFrm.pack(side='bottom')
        

    def okBtn_click(self):
        self.btn_flag=1
        self.batchinf_b.setData([int(self.batchNo.get()),int(self.noperBatch.get()),int(self.noperPacket.get()),int(self.speed.get())])
        self.topWin.destroy()      
        
    def cancelBtn_click(self):
        self.btn_flag=0
        self.topWin.destroy()

    def winPo(self):
        self.rootx=self.root.winfo_rootx()
        self.rooty=self.root.winfo_rooty()
        strFormat=("%dx%d+%d+%d"%(self.tt[0],self.tt[1],(self.rootx+self.tt[2]),(self.rooty+self.tt[3])))
        return strFormat
    def okcancelHide(self):#hide ok & cancel button
        self.okcancelFrm.pack_forget()
    
    def okcancelShow(self): #show ok cancel button
        self.topWin.geometry(self.par)
        self.okcancelFrm.pack(side='bottom')
    def destroyObj(self):
        self.topWin.destroy()
"""
define batch class
"""
class batchWin(popupWin):
    def __init__(self,parent,title,object0,speed,*args,**kwargs):
        super().__init__(parent,title,object0,speed,*args,**kwargs)
        '''
        self.batchNo = TK.StringVar()
        self.batchNo.set("1") #default set 1 batch
        self.noperBatch =TK.StringVar()
        self.noperBatch.set("1") #default set 10
        self.noperPacket = TK.StringVar()
        self.noperPacket.set("100") #default set 50 kernel for one packet
        self.speed = TK.StringVar()
        self.speed.set(speed)
        '''
        #numpadShow_Flag show--1, off--0
        self.numpadShow_Flag=0

        #create entrys
        self.batchFrm=TK.Frame(self.topWin,bg='cadet blue')

        self.batchnoLbl=TK.Label(self.batchFrm,text="1. Batch No ",
                                width=13,
                                bg='gray',font=self.myFont
                                )
        self.batchnoLbl.grid(row=0,column=0,pady=10,padx=30,ipady=10)
        self.batchnoLblN = TK.Label(self.batchFrm,textvariable=self.batchNo,font=self.myFont,relief='ridge')
        self.batchnoLblN.bind("<Button-1>",self.mouse_click)
        self.batchnoLblN.grid(row=0,column=1,pady=5,padx=10,ipady=10)
       
        self.speedLbl=TK.Label(self.batchFrm,text="2. Speed",
                                width=13,
                                bg='gray',font=self.myFont
                                )
        self.speedLbl.grid(row=1,column=0,pady=10,padx=10,ipady=10)
        self.speedEntry = TK.Entry(self.batchFrm,textvariable=self.speed,font=self.myFont,width=5)
        self.speedEntry.bind("<Button-1>",self.mouse_click)
        self.speedEntry.grid(row=1,column=1,pady=10,padx=10,ipady=10)
        
        self.noperbatchLbl=TK.Label(self.batchFrm,text="3. No./Batch", 
                                    width=13,
                                    bg='gray',font=self.myFont)
        self.noperbatchLbl.grid(row=2,column=0,pady=10,padx=30,ipady=10)
        self.noperbatchEntry=TK.Entry(self.batchFrm,textvariable=self.noperBatch,font=self.myFont,width=10)
        self.noperbatchEntry.bind("<Button-1>",self.mouse_click) #call numpad
        self.noperbatchEntry.grid(row=2,column=1,pady=10,padx=30,ipady=10)

        self.seednoLbl = TK.Label(self.batchFrm,text="4. No./Packet",
                                  width=13,
                                  bg='gray',font=self.myFont)
        self.seednoLbl.grid(row=3,column=0,pady=10,padx=30,ipady=10)
        self.seednoEntry= TK.Entry(self.batchFrm,textvariable=self.noperPacket,font=self.myFont,width=10)
        self.seednoEntry.bind("<Button-1>",self.mouse_click) #call numpad
        self.seednoEntry.grid(row=3,column=1,pady=20,padx=30,ipady=10)
        self.batchFrm.pack(side='top')
        '''
        self.noperbatchEntry.select_range(0,'end')
        self.noperbatchEntry.focus()
        '''
        self.speedEntry.select_range(0,'end')
        self.speedEntry.focus()
        self.topWin.wait_window()

    def mouse_click(self,event):#create a numpad
        if self.numpadShow_Flag==0:
            self.numpadShow_Flag =1

            #resize window size
            self.topWin.geometry("%dx%d+%d+%d"%(self.tt[0],(175+self.tt[1]),
                                (self.rootx+self.tt[2]),
                                (self.rooty+self.tt[3]))
                                )
       
            self.myFont=Font(family="Times Newn Roman",size=20)
            #invisable ok and cancel button
            self.okcancelHide()

            #create numpad
            self.numpadFrm=TK.Frame(self.topWin,bg='gray',bd=2,relief='ridge')
            self.numpadFrm.pack(fill='x',padx=14)
             
            btn_list = ['6','7',  '8',  '9', '0','1','2','3','4','5', '<--','Tab', 'Next\n Batch','OK','Cancel']
            # create and position all buttons with a for-loop
            # r, c used for row, column grid values
            r = 0
            c = 0
            n = 0
            # list(range()) needed for Python3
            btn = []
            for label in btn_list:
                # partial takes care of function and argument
                cmd = lambda x = label: self.click(x)
                # create the button
                cur = TK.Button(self.numpadFrm, text=label, bg='gray',
                                font=self.myFont,
                                width=4,height=2,
                                command=cmd
                                )

                btn.append(cur)
                # position the button         
                btn[-1].grid(row=r, column=c,ipadx=5,ipady=3)
 
                # update row/column position
                c += 1
                if c == 5:
                    c = 0
                    r += 1

    def click(self,label):
        print(label)
        if label == '<--':
            if self.topWin.focus_get()==self.speedEntry:
                currentText = self.speed.get()
                self.speed.set(currentText[:-1])   #remove last character
                self.speedEntry.icursor('end')
            elif self.topWin.focus_get()==self.noperbatchEntry:
                currentText = self.noperBatch.get()
                self.noperBatch.set(currentText[:-1])
                self.noperbatchEntry.icursor('end')
            elif self.topWin.focus_get()==self.seednoEntry:
                 currentText = self.noperPacket.get()
                 self.noperPacket.set(currentText[:-1])
                 self.seednoEntry.icursor('end')
            else:
                return
                 
        elif label == 'OK':
            if self.noperBatch.get() =="" or self.noperPacket.get() =="" or self.noperBatch.get() =="0" or self.noperPacket.get() =="0":
                Message_dialogue(self.topWin,'Please enter number !','ok','Reminder !')
            elif self.speed.get()=="" or int(self.speed.get())>255 or self.speed.get()=="0": 
                Message_dialogue(self.topWin,'Invalid speed !(1~255)','ok','Reminder!')
            else:
                self.batchinf_b.setData([int(self.batchNo.get()),int(self.noperBatch.get()),int(self.noperPacket.get()),int(self.speed.get())])
                #close numpad
                self.numpadShow_Flag=0
                self.numpadFrm.destroy()
                self.btn_flag=1
                self.topWin.destroy()
        elif label=='Cancel':
             #close numpad
            self.numpadShow_Flag=0
            self.numpadFrm.destroy()
            self.okcancelShow()

        elif label=='Next\n Batch':#do nothing
            if self.noperBatch.get()=="" or self.noperPacket.get()=="":
                Message_dialogue(self.topWin,'Please enter Number !','ok','Reminder !')
            else:
                self.batchinf_b.setData([int(self.batchNo.get()),int(self.noperBatch.get()),
                                         int(self.noperPacket.get()),int(self.speed.get())])
                bt=int(self.batchNo.get())+1
                self.batchNo.set(str(bt))
               
                self.noperbatchEntry.select_range(0,'end')
                self.noperbatchEntry.focus()
                #self.noperbatchEntry.icursor(0)
        elif label=='Tab':
            if self.topWin.focus_get() == self.speedEntry:
                self.noperbatchEntry.select_range(0,'end')
                self.noperbatchEntry.focus()
            elif self.topWin.focus_get()==self.noperbatchEntry:
                self.seednoEntry.select_range(0,'end')
                self.seednoEntry.focus()
            elif self.topWin.focus_get()==self.seednoEntry:
                 self.speedEntry.select_range(0,'end')
                 self.speedEntry.focus()
            else:
                return
        else:
            if self.topWin.focus_get() == self.speedEntry:
                if self.speedEntry.select_present():
                    self.speedEntry.select_clear()
                    self.speed.set(label)
                else:
                    currentText=self.speedEntry.get()
                    self.speed.set(currentText+label)
                self.speedEntry.icursor('end') 
                                   
            elif self.topWin.focus_get()==self.noperbatchEntry:
                if self.noperbatchEntry.select_present():
                    self.noperbatchEntry.select_clear()
                    self.noperBatch.set(label)
                else:
                    currentText = self.noperbatchEntry.get()
                    self.noperBatch.set(currentText+label)
                self.noperbatchEntry.icursor('end')
                
            elif self.topWin.focus_get()==self.seednoEntry:
                if self.seednoEntry.select_present():
                    self.seednoEntry.select_clear()
                    self.noperPacket.set(label)
                else:
                    currentText = self.seednoEntry.get()
                    self.noperPacket.set(currentText+label)
                self.seednoEntry.icursor('end') 
            else:
                return


"""
    define main page
"""
class mainPage(frlsWin):   
    def __init__(self):
        super().__init__()
        self.page_Index=''

        #create and layout buttons 
        #create a frame for list button
        self.btnFrame=TK.Frame(self.root,bg="SlateBlue3",bd=1,relief='ridge')
        self.btnFrame.pack(fill=TK.BOTH,pady=20, padx=20)
        self.myFont=Font(family="Times Newn Roman",size=30)

        self. LoadfileBtn1 = TK.Button(self.btnFrame,text='Load File...',
                                        width=18,bg='gray',relief=TK.RAISED,
                                        font=self.myFont,
                                        command=self.loadfile_click)
        self.LoadfileBtn1.grid(column=0,row=0,padx=80, pady=60,ipady=10)
    
        self.indepBtn1 = TK.Button(self.btnFrame,text= "Independent Mode",
                                        width=18,bg="gray",relief=TK.RAISED,
                                        font=self.myFont, 
                                        command=self.independent_click)
        self.indepBtn1.grid(column=0,row=1,padx=80,pady=60,ipady=10)

        self.counterstateBtn1 = TK.Button(self.btnFrame,text="Counter state",
                                        width=18,bg="gray",relief=TK.RAISED,
                                        font=self.myFont, 
                                        command=self.counterstate_click)
        self.counterstateBtn1.grid(column=0,row=2,padx=80,pady=60,ipady=10)

        
        self.remoteBtn =TK.Button(self.btnFrame,text="RS232 Mode",
                                width=18,bg="gray",relief=TK.RAISED,
                                font=self.myFont,
                                command=self.RS232_click)
        self.remoteBtn.grid(column=1,row=0,padx=20,pady=60,ipady=10)

        self.networkBtn1 =TK.Button(self.btnFrame,text="RS485 TCP",
                                width=18,bg="gray",relief=TK.RAISED,
                                font=self.myFont,
                                command=self.RS485tcp_click)
        self.networkBtn1.grid(column=1,row=1,padx=20,pady=60,ipady=10)

        self.networkBtn2 =TK.Button(self.btnFrame,text="RS485 RTU",
                                width=18,bg="gray",relief=TK.RAISED,
                                font=self.myFont,
                                command=self.RS485rtu_click)
        self.networkBtn2.grid(column=1,row=2,padx=20,pady=60,ipady=10)
        #exit program
        self.root.bind_all('<Escape>',self.exitProg)
    
        self.root.mainloop()
        logging.info('Exit to desktop !')
        page_Back='back'
   
    def RS232_click(self): 
        self.page_Index = "1"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()

    def independent_click(self):
        self.page_Index = "2"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()

    def counterstate_click(self):
        self.page_Index="3"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()
    def loadfile_click(self):
        self.page_Index="4"
        mess=Message_dialogue(self.root,'Packet ID requirred ?','yes-no','Selection')
        if mess.iflag=='OK': #required
            self.page_Index="4-y"
        else :
            self.page_Index="4-n"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()
    def RS485tcp_click(self):
        self.page_Index="5"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()
    def RS485rtu_click(self):
        self.page_Index="6"
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()
    def exitProg(self,event): 
        global page_Back
        page_Back='back'
        self.powerThread.join() #wait thread exit
        page_Back=''
        self.root.destroy()
        self.page_Index=""

"""
    define a class for remote mode rs232,rs485
    start:Sept 21, 2021, Daniel T
"""
class mainPage_010(frlsWin):
    def __init__(self,valStr):
        super().__init__()
        # import serial class
        import serial_esc2 as serial_esc2

        self.serialType=valStr
        '''
        rs485, counter_Spe[0], seed number, 
        counter_Spe[1]:status of the counte
            counter status:0-waiting, 1-totalize, 2-batch
        counter_Spe[0]:counter number
        '''
        '''
        logging format
        
        file_log = open('Esc2_data.log','w').close() #close log file
        format1 = '%(levelname)s--%(asctime)s--%(message)s'
        logging.basicConfig(filename='Esc2_data.log',
                            filemode='a',
                            format = format1,
                            level = logging.INFO,
                            datefmt = '%H:%M:%S')
        '''
        self.counter_Spe=[0,0] 
        self.lab=None# for common label
        self.listen_Flag=True #rs232 listen thread flag
        self.tempMode='idle' #previous count mode

        #Counter start to counter by create a new thread
        self.itemTv_id=[] #empty list
        self.itemlist=0   #list number
        self.treeTop=0  #initial position
        self.tNo=0  #current count number
        
        self.x0=0
        self.y0=0
        self.xm=0
        self.ym=0
        self.xr=0
        self.yr=0

        #define rs232 variable 
        self.rs232_Command=''   #comand Start, Disconnect, Dump
        self.rs232_Command_p='' #composite command
        self.Crop=''    #corp name
        self.rs232_Speed='50'
        self.h_speed=0
        self.l_Speed=0
        self.tNo=0  #current count number

        #setup font
        self.myFont=Font(family="Times Newn Roman",size=20)
        
        #create displayFrm
        self.displayFrm = TK.Frame(self.root,bg='gray')
        self.displayFrm.pack(fill=TK.X,pady=10,padx=40)

        #create a speed label
        self.rsSpeedLbl=TK.Label(self.displayFrm,text='Speed: \n50',
                                font=self.myFont,
                                bg='gray'
                                )
        self.rsSpeedLbl.pack(side=TK.LEFT,padx=10,ipady=5,ipadx=5)

        #create a batch label
        self.rsBatchLbl=TK.Label(self.displayFrm,text='BatchNo:\n1',
                                font=self.myFont,
                                bg='gray'
                                )
        self.rsBatchLbl.pack(side=TK.LEFT,padx=35,ipady=5,ipadx=5)
        
        #create a packet label
        self.rsPacketLbl = TK.Label(self.displayFrm,text='Packet: \n1',
                                    font = self.myFont,
                                    bg='gray'
                                    )
        self.rsPacketLbl.pack(side=TK.LEFT,padx=35,ipady=5,ipadx=5)

        #create an input Label
        self.countText=TK.IntVar()
        self.countText.set(0)
        self.countLbl_Num = TK.Label(self.displayFrm,textvariable=self.countText,
                                     bg='dark khaki',
                                     font=Font(family='Times New Roman',size=26)
                                     )
        self.countLbl_Num.pack(side=TK.RIGHT,pady=1,padx=20,ipady=6)

        #create count label
        self.rsCount = TK.Label(self.displayFrm,text='Count',
                                font=self.myFont,
                                bg='gray'
                                )
        self.rsCount.pack(side= TK.RIGHT,padx=20,pady=1,ipady=5,ipadx=5)
        
        #create treeviewFrm
        self.treeviewFrm = TK.Frame(self.root,bg='gray')

        self.treeview_rs232()
        self.treeviewFrm.pack(fill=TK.X,pady=10,padx=40)

        #create and layout button
        self.myFont=Font(family="Times Newn Roman",size=24)
        self.backBtn =TK.Button(self.root,text="Cancel",
                                  bg="gray", relief=TK.RAISED,
                                  font=self.myFont,
                                  command=self.cancel_click)
        self.backBtn.pack(pady=10,ipadx=5,ipady=10)

        self.cMode=count_mode[0]
        
        self.isRs_data_in=False
        if self.serialType=='rs232':
             #initail selial class for rs232
            self.serial = serial_esc2.serialEsc('rs232')
            self.commLabel('Waiting for connect...') 
            self.rsListen()
        elif self.serialType=='rs485tcp':
            self.commLabel('Waiting for command...') 
            import subprocess
            localhost= str(subprocess.check_output(['hostname', '-I'])).split(' ')[0].replace("b'","")
            #localhost='192.168.2.35'
            port=502
            self.rs485tcp=ModbusServerRS(localhost,port)
        
            self.rsListen_rs485tcp()
            
        else:
             #initail selial class for rs485rtu
            self.serial = serial_esc2.serialEsc('rs485rtu')
            self.commLabel('Waiting for command...')
            self.rsListen_rs485rtu()

        self.root.mainloop()

    def __def__(self):
        del self.serial
        if self.serialType=='rs485tcp':
            del self.rs485tcp
    
    #treeview 
    def treeview_rs232(self):
        #create a batch treeview
        #set attribute of display
        self.style=TTK.Style()
        self.style.theme_use('alt')
        #set row height
        self.style.configure('TC.Treeview',rowheight=40,
                            font=('Times',24),
                            background='silver',
                            foreground='black'
                            )
        #set heading size
        self.style.configure('TC.Treeview.Heading',
                            font=('Times',18),
                            background='silver',
                            foreground='black'
                            )
        #set layout
        self.style.layout("myStle.Treeview",
                        [('myStyle.Treeview.treearea',{'sticky':'nswe'})]
                        )

        #set select color strip
        self.style.map('Treeview',background=[('selected','blue')])
        
        #define vertical scrollbar
        self.style.element_create('My.Vertical.TScrollbar.trough','from','clam')
        self.style.element_create('My.Vertical.TScrollbar.thumb','from','clam')
        self.style.element_create('My.Vertical.TScrollbar.grip','from','clam')
        self.style.configure('My.Vertical.TScrollbar',arrowsize=30)
        scrobar_Treeview=TTK.Scrollbar(self.treeviewFrm,orient='vertical',style='My.Vertical.TScrollbar') 
        
        #create a treeview for batch count
        self.itemTv_Rs = TTK.Treeview(self.treeviewFrm,
                                        columns=("Source_ID","Crop","Packet","No/Packet","Remainder","Comment"),
                                        style='TC.Treeview', 
                                        height=11)
        scrobar_Treeview.config(command=self.itemTv_Rs.yview)
        self.itemTv_Rs.config(yscrollcommand=scrobar_Treeview.set)
        scrobar_Treeview.pack(side='right',fill='y')

        self.itemTv_Rs.tag_configure('bg',background='sky blue')
        self.itemTv_Rs.tag_configure('bg-1',background='steel blue')
        self.itemTv_Rs.tag_configure('bg-2',background='dimgray') #finsihed color

        self.itemTv_Rs.heading("#0",text="No",anchor=TK.W)
        self.itemTv_Rs.heading("Source_ID",text="Source_ID",anchor=TK.CENTER)
        self.itemTv_Rs.heading("Crop",text="Crop",anchor=TK.CENTER)
        self.itemTv_Rs.heading("Packet",text="Packet",anchor=TK.CENTER)
        self.itemTv_Rs.heading("No/Packet",text="No/Packet",anchor=TK.CENTER)
        self.itemTv_Rs.heading("Remainder",text="Remainder",anchor=TK.CENTER)
        self.itemTv_Rs.heading("Comment",text="Comment",anchor=TK.CENTER)
        #format column
        self.itemTv_Rs.column("#0",width=80,stretch=TK.NO,anchor=TK.W)
        self.itemTv_Rs.column("Source_ID",width=250,stretch=TK.YES,anchor=TK.CENTER)
        self.itemTv_Rs.column("Crop",width=140,stretch=TK.YES,anchor=TK.CENTER)
        self.itemTv_Rs.column("Packet",width=100,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Rs.column("No/Packet",width=180,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Rs.column("Remainder",width=150,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Rs.column("Comment",width=240,stretch=TK.NO,anchor=TK.CENTER)
        
        self.itemTv_Rs.pack(fill=TK.X)

        self.itemTv_Rs.bind('<Button-1>',self.mouse_Pressed)
        self.itemTv_Rs.bind('<B1-Motion>',self.mouse_Move,add='+')
        self.itemTv_Rs.bind('<ButtonRelease-1>',self.mouse_Release,add='+')
        
    #change count display
    def set_countText(self,val) :
         self.countText.set(val)
     #set batch speed
    def mouse_Pressed(self,event):
            self.x0,self.y0=event.x,event.y
            #self.widget_name=self.root.focus_get()
            #print("Pressed:x={},y={}".format(self.x0,self.y0))
    #mouse is motion
    def mouse_Move(self,event):
         self.xm,self.ym=event.x,event.y
         #print("motion:x={},y={}".format(self.xm,self.ym))
         gap=self.y0-self.ym
         no=divmod(abs(gap),26)
         rowNo=int(no[0])
         if rowNo >1:
            if self.itemlist :
                if gap < 0: #slide downward
                    nol=self.treeTop-rowNo
                    if nol>=0:
                        self.treeTop -=rowNo 
                    else:
                        self.treeTop=0
                        self.itemTv_Rs.see(str(self.treeTop)) 
                      
                else: #slide upward
                    l=self.treeTop+11
                    nol=11-l
                    if nol>rowNo:
                        self.treeTop +=rowNo
                        self.itemTv_Rs.see(str(l+rowNo))
                        
                    elif nol>=0:
                        self.treeTop +=nol
                        self.itemTv_Rs.see(str(self.itemlist-1))
    #mouse is releaseed
    def mouse_Release(self,event):
         self.xr,self.yr=event.x,event.y
         #print("Release:x={},y={}".format(self.xr,self.yr))
         gap=self.y0-self.yr
         no=divmod(abs(gap),26)
         rowNo=int(no[0])
         if rowNo >1:
            if self.itemlist >11:
                if gap < 0: #slide downward
                    nol=self.treeTop-rowNo
                    if nol>=0:
                        self.treeTop -=rowNo 
                    else:
                        self.treeTop=0
                        self.itemTv_Rs.see(str(self.treeTop)) 
                    
                else: #slide upward
                    l=self.treeTop+16
                    nol=11-l
                    if nol>rowNo:
                        self.treeTop +=rowNo
                        self.itemTv_Rs.see(str(l+rowNo))  
                        
                    elif nol>=0:
                        self.treeTop +=nol
                        self.itemTv_Rs.see(str(self.itemlist-1))     
    #update top number of list in treeview
    def update_Treetop(self):
         no=self.itemlist-11
         if no<=0 :
            self.treeTop=0
         else:
            self.treeTop=no 
     
    def getTreeviewNo(self):
         return self.tNo

    def setTreeviewNo(self,No_batch,lBatch_No):
         self.tNo=self.itemlist-No_batch+lBatch_No-1
    #define dump Thread
    def dump_Thread(self,str_):
        lab=TK.Label(self.root,text=str_,
                    bg='lightblue',
                    fg='darkred',
                    font=('Times',30))
        
        lab.place(x=350,y=250,height=60,width=250)
        eventThread.wait()
        lab.destroy()
     
     #prepareData
    def prepareData(self,byteStr):
        iFlag=True
        try:
            if chr(byteStr[1])=='C' or chr(byteStr[1])=='c':
                if chr(byteStr[3])=='M' or chr(byteStr[3])=='m':
                    i=0
                    #extract speed
                    f_index=4
                    No2=byteStr.find(b'\x03')
                    len=No2-f_index
                    temStr=bytearray(len)
                
                    for i in range(len):    
                        temStr[i]=byteStr[f_index+i]
                    
                    self.rs232_Speed=str((temStr.decode()))
                    self.rs232_Command='M'
                           
                else:
                    i=0
                    #extract command
                    f_index=3
                    No2=byteStr.find(b'\x03')
                    len=No2-f_index
                    temStr=bytearray(len)
                
                    for i in range(len):    
                        temStr[i]=byteStr[f_index+i]
                    
                    self.rs232_Command=str(temStr.decode())
                        
            elif chr(byteStr[1])=='P' or chr(byteStr[1])=='p':
                obj_batchInf.removeData() #clear data
                #extract crop
                No1=3
               
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)

                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.Crop=str(temStr.decode())
                
                #extract batch Number           
                No1=No2+1
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                    
                obj_batchInf.batch_noSet(int(temStr.decode()))
                No= int(temStr.decode())       
                
                for j in range(No):
                    #extract packet number
                    No1=No2+1
                    No2=byteStr.find(b'\x1f',No1)
                    len=No2-No1
                    temStr=bytearray(len)
                    for i in range(len):    
                        temStr[i]=byteStr[No1+i]

                    obj_batchInf.no_perpacketSet(int(temStr.decode()))
                        
                    
                    #extract packet number
                    No1=No2+1
                    No2=byteStr.find(b'\x1f',No1)
                    len=No2-No1
                    temStr=bytearray(len)
                    for i in range(len):    
                        temStr[i]=byteStr[No1+i]
                
                    obj_batchInf.no_perbatchSet(int(temStr.decode()))

                # extract hori_gat
                No1=No2+1
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.hori_gate=int(temStr.decode())
                          
                # extract vertical_gate
                No1=No2+1
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.ver_gate=int(temStr.decode())
                    
                # extract snsitivity
                No1=No2+1
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.sensiStr=int(temStr.decode())
                        
                # extract speed
                No1=No2+1
                No2=byteStr.find(b'\x1f',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.rs232_Speed=str((temStr.decode()))      

                # extract command
                No1=No2+1
                No2=byteStr.find(b'\x03',No1)
                len=No2-No1
                temStr=bytearray(len)
                for i in range(len):    
                    temStr[i]=byteStr[No1+i]
                
                self.rs232_Command_p=str((temStr.decode()))
                                
            else:
                return False   
        except Exception:
            logging.info('main: Rs232 error')
            Message_dialogue(self.root,'Abnormal data\nCheck input data !','ok','Reminder !')
            iFlag=False
        finally:
            return iFlag
    def prepareData485_tcp(self,typeF,data):
        iFlag=True
        try:
            if typeF==100:#totalize
                obj_batchInf.batch_noSet(0)
                obj_batchInf.no_perpacketSet(0)
                obj_batchInf.no_perbatchSet(0)
                speed=int(data[0])
                self.rs232_Speed=str(speed)
            
                self.cMode=count_mode[1]
                self.rs485tcp.sendData(60,[1])

            elif typeF==200: #batchInf
                obj_batchInf.removeData() #clear data
                obj_batchInf.batch_noSet(1)
                noP=int(data[0])
                obj_batchInf.no_perbatchSet(noP)
                noP=int(data[1])
                obj_batchInf.no_perpacketSet(noP)
                speed=int(data[2])
                self.rs232_Speed=str(speed)
                self.cMode=count_mode[2]
                self.rs485tcp.sendData(60,[2])
                
        except Exception:
            Message_dialogue(self.root,'Abonormal message !','ok','Warning')
            iFlag=False
        finally:
            return iFlag
           
    def prepareData485_rtu(self,data):
        typeF=int(data[3])
        iFlag=True
        try:
            if typeF==100:#totalize
                obj_batchInf.batch_noSet(0)
                obj_batchInf.no_perpacketSet(0)
                obj_batchInf.no_perbatchSet(0)
                speed=int(data[8])
                self.rs232_Speed=str(speed)
                self.counter_Spe[1]=0x01    #totalize

            elif typeF==200: #batchInf
                obj_batchInf.removeData() #clear data
                obj_batchInf.batch_noSet(1)
                noP=int(data[7])*256+int(data[8])
                obj_batchInf.no_perbatchSet(noP)
                noP=int(data[9])*256+int(data[10])
                obj_batchInf.no_perpacketSet(noP)
                speed=int(data[11])*256+int(data[12])
                self.rs232_Speed=str(speed)
                self.counter_Spe[1]=0x02 #batch count
        except Exception:
            Message_dialogue(self.root,'Abonormal message !','ok','Warning')
            iFlag=False
        finally:
            return iFlag
        
    def rsListen(self):        
         logging.info("Main : create a thread of rslisten_thread")
         self.rsThread_02=threading.Thread(target=self.rslisten_Thread, 
                                            name="rs232Listen",args=('Wating for connect...',)
                                            )                   
         self.rsThread_02.setDaemon(True)
         self.rsThread_02.start()
         time.sleep(1)

         logging.info('Main :Thread rs232Listen has been created')
         self.rsThread_03=threading.Thread(target=self.rscount_Thread, 
                                          name="rsCount",args=(1,)
                                         ) 
         self.rsThread_03.setDaemon(True)
         logging.info("Main : Thread rsCount has been created")
         self.rsThread_03.start() 

    def rsListen_rs485tcp(self):
         #start rs485 tcp
         logging.info("Main : create a thread of rslisten_thread485_tcp")
         self.rsThread_rs485tcp=threading.Thread(target=self.rslisten_Thread485_tcp, 
                                            name="rsListen_485tcp",args=('Wating for command...',)
                                            )                   
         self.rsThread_rs485tcp.setDaemon(True)
         self.rsThread_rs485tcp.start()
         time.sleep(1)
         
         logging.info('Main :Thread rs232Listen has been created')
         self.rsThread_03=threading.Thread(target=self.rscount_Thread, 
                                          name="rsCount",args=('rs485',)
                                         ) 
         self.rsThread_03.setDaemon(True)
         logging.info("Main : Thread rsCount has been created")
         self.rsThread_03.start()         
        
    def rsListen_rs485rtu(self):
         #enalbe rs485 and disable rs232
         rs485Set.read485()
         logging.info("Main : create a thread of rslisten_thread485_rtu")
         self.rsThread_rs485rtu=threading.Thread(target=self.rslisten_Thread485_rtu, 
   
                                            name="rsListen_485rtu",args=('Wating for command...',)
                                            )                   
         self.rsThread_rs485rtu.setDaemon(True)
         self.rsThread_rs485rtu.start()
         time.sleep(1)

         logging.info('Main :Thread rs232Listen has been created')
         self.rsThread_03=threading.Thread(target=self.rscount_Thread, 
                                          name="rsCount",args=('rs485',)
                                         ) 
         self.rsThread_03.setDaemon(True)
         logging.info("Main : Thread rsCount has been created")
         self.rsThread_03.start() 

    def rslisten_Thread(self,strV):  
         connect_Flag=False
         while self.listen_Flag:
            if not connect_Flag:
                if self.serial.Ping():
                    valve.Closevalve()
                    time.sleep(0.25)
                    valve.Openvalve()
                    self.lab.config(text='Waiting for data...')
                    connect_Flag=True
                #time.sleep(0.1)
            else:  
                time.sleep(0.1)
                byteStr=self.serial.readData()
                if byteStr:
                    logging.info(byteStr)
                    if self.prepareData(byteStr) :
                        if self.rs232_Command_p == 'Start' or self.rs232_Command_p =='start':  
                            self.rs232_Command=''
                            self.itemAdd()
                            self.speed_Change(self.rs232_Speed)
                            self.rs232_Command_p=''
                            if self.lab:
                                self.lab.destroy()
                                self.lab=None 
                        elif  self.rs232_Command== 'Start' or self.rs232_Command=='start': 
                            if(self.tempMode==count_mode[2]) and self.cMode==count_mode[0]:
                                self.itemAdd()
                                if valve.state_valve()==1 :
                                    valve.Openvalve()
                                self.rs232_Command=''
                                self.tempMode=count_mode[0]
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None  
                                self.speed_Change(self.rs232_Speed)
                                
                        elif  self.rs232_Command== 'Recycle' or self.rs232_Command=='recycle': 
                            if(self.tempMode==count_mode[2]) and self.cMode==count_mode[0]:
                                self.itemAdd()
                                if valve.state_valve()==1 :
                                    valve.Openvalve()
                                self.rs232_Command=''
                                self.tempMode=count_mode[0]
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None  
                                self.speed_Change(self.rs232_Speed) 
                        elif self.rs232_Command == 'Disconnect' or self.rs232_Command =='disconnect':
                            self.cMode=count_mode[0]
                            if self.lab:
                                self.lab.config(text='Waiting for connect...')
                            else:
                                self.commLabel('Waiting for connect...')
                            turnTable.stop_Turntab()
                            connect_Flag=False
                            self.rs232_Command =''
                            self.serial.Flag=0
                            
                        elif self.rs232_Command =='M':
                            self.speed_Change(self.rs232_Speed)
                            if rs232_carousel =='1':
                                turnTable.stop_Turntab()
                            self.rs232_Command=""
                        elif self.rs232_Command =='Zero' or self.rs232_Command =='zero':
                            if self.cMode==count_mode[0] and self.tempMode==count_mode[0]:
                                count.clearCount()
                                self.set_countText(0)
                                
                            elif self.tempMode==count_mode[2]:
                                count.clearCount()
                                self.set_countText(0)
                                self.rs232_Command=''
                            
                        elif self.rs232_Command =='Stop' or self.rs232_Command =='stop':
                            if self.cMode==count_mode[0]:
                                turnTable.stop_Turntab()
                        
                        elif self.rs232_Command == 'Dump' or  self.rs232_Command == 'dump':
                            #thread is at idle state
                            self.cMode = count_mode[4]  #set dumpping count mode 
                            self.tempSpeed=self.rs232_Speed 
                            if self.lab:
                                self.lab.config(text='Cleanout...')
                            else:
                                self.commLabel('Cleanout...')
                    
                             #turntable start to rotating
                        
                            strSpeed=f'Speed: \n200'
                            self.rsSpeedLbl.config(text=strSpeed)
                            self.rs232_Command=""

                    byteStr=0 #clear      

         logging.info("Main : rsThread_02 thread has been terminated")
    def rslisten_Thread485_tcp(self,strV):
        try:
            logging.info('Start server')
            self.rs485tcp.start()
            logging.info('Server is online !')
            self.rs485_flag=True
            #clear  memory
            self.rs485tcp.clear_memo()
             
            while self.rs485_flag:
                if (self.rs485tcp.readData(100,1))[0] !=0: #read totalize
                    '''
                     function code:16(0X10)
                     Address:100
                     data:speed
                    '''
                    data=self.rs485tcp.readData(100,1)
                    if self.prepareData485_tcp(100,data):#speed
                        self.itemAdd()
                        time.sleep(0.5)
                        self.speed_Change(self.rs232_Speed)
                        if self.lab:
                            self.lab.destroy()
                            self.lab=None
                    print('Totalize !')
                    self.rs485tcp.sendData(100,[0])
                elif (self.rs485tcp.readData(200,1))[0] !=0:#read batch
                    '''
                     function code:16(0X10)
                     Address:200
                     data:package No,No of per-package, speed
                    '''
                    data=self.rs485tcp.readData(200,3)
                    if self.prepareData485_tcp(200,data):
                        self.itemAdd()
                        time.sleep(0.5)
                        self.speed_Change(self.rs232_Speed)
                        if self.lab:
                            self.lab.destroy()
                            self.lab=None

                    print('Batch Count !')
                    self.rs485tcp.sendData(200,[0,0,0])
                elif (self.rs485tcp.readData(10,1))[0] !=0:#valve command
                    '''
                     function code:16(0x10)
                     Address=10
                     data:1--close, 2--open
                    '''
                    if self.cMode !=count_mode[2]:
                        data= self.rs485tcp.readData(10,1)
                        if data[0]==1: #close valve
                            valve.Closevalve()
                        elif data[0]==2:    #open valve
                            valve.Openvalve()
                        print('Valve command !')
                        self.rs485tcp.sendData(10,[0])
                elif (self.rs485tcp.readData(12,1))[0] !=0:#speed command
                    '''
                     function code:16(0x10)
                     Address=12
                     data:speed
                    '''
                    data= self.rs485tcp.readData(12,1)
                    self.rs232_Speed=str(data[0])
                    self.speed_Change(self.rs232_Speed)
                    self.rs232_Command=''
                    print('Speed command')
                    self.rs485tcp.sendData(12,[0])
                elif (self.rs485tcp.readData(15,1))[0] !=0:#dump command
                    '''
                     function code:16(0x10)
                     Address=15
                     data:dump speed
                    '''
                    data= self.rs485tcp.readData(15,1)
                    #thread is at idle state
                    self.cMode = count_mode[4]  #set dumpping count mode 
                    self.tempSpeed=self.rs232_Speed 
                    if self.lab:
                        self.lab.config(text='Dumpping...')
                    else:
                        self.commLabel('Dumpping...')
            
                        #turntable start to rotating
                
                    strSpeed=f'Speed: \n200'
                    self.rsSpeedLbl.config(text=strSpeed)
                    self.rs232_Command=""
                    print('dump command !')
                    self.rs485tcp.sendData(15,[0])
                
                elif (self.rs485tcp.readData(20,1))[0] !=0:#start command
                    '''
                     function code:16(0x10)
                     Address=20
                     data:start
                    '''
                    data= self.rs485tcp.readData(20,1)
                    if self.cMode==count_mode[1] :
                        self.rs232_Command='Start'
                        if self.lab:
                            self.lab.destroy()
                            self.lab=None
                    elif self.cMode==count_mode[0] and (self.tempMode==count_mode[2] or self.tempMode==count_mode[1]) :
                        self.itemAdd()
                        self.speed_Change(self.rs232_Speed)              
                        if self.lab:
                            self.lab.destroy()
                            self.lab=None
    
                    self.rs485tcp.sendData(20,[0])
                    
                elif (self.rs485tcp.readData(25,1))[0] !=0: #stop command
                    '''
                     function code:16(0x10)
                     Address=25
                     data:stop
                    '''
                    print('stop command')
                    data= self.rs485tcp.readData(25,1)
                    if self.cMode==count_mode[1]:
                        self.rs232_Command='Stop'
                    elif self.cMode==count_mode[0]:
                        turnTable.stop_Turntab()
                        
                    self.rs485tcp.sendData(25,[0])
                    
                elif (self.rs485tcp.readData(30,1))[0] !=0:#zero command
                    '''
                     function code:16(0x10)
                     Address=30
                     data:zero
                    '''
                    if self.cMode==count_mode[1]:
                        self.rs232_Command='Zero'
                    elif self.cMode==count_mode[0]:
                        count.clearCount() 
                        self.set_countText(0)   
                    self.rs485tcp.sendData(50,[0])
                elif (self.rs485tcp.readData(35,1))[0] !=0: #Done command
                    '''
                     function code:16(0x10)
                     Address=35
                     data:Done
                    '''
                    if self.cMode==count_mode[1] or self.cMode==count_mode[2] :
                        self.rs232_Command='Reset'
                        if self.lab:
                            self.lab.destroy()
                            self.lab=None
                    self.rs485tcp.sendData(35,[0])
                elif (self.rs485tcp.readData(80,1))[0] !=0:#read kernal back to plc
                    '''
                     function code:4(0x04)
                     Address=50; Address=80 is request address
                     data:request to obtain seed number
                    '''
                    t=self.counter_Spe[0] #seed number  
                    self.rs485tcp.sendData(50,[t])   
                time.sleep(0.5)
            #exit tcp
            self.rs485tcp.stop()
            print('Online off')
        except Exception:
            logging.info('Shutdown server...')
            self.rs485tcp.stop()
            logging.info('Server is offline')
            if self.lab:
                self.lab.config(text='Start server failed !')
            else:
                self.commLabel('Start server failed !')

    def rslisten_Thread485_rtu(self,strV):
         while self.listen_Flag:
            data=self.serial.readData485()
            if data[0]==0x10: #check address
                #crc16 check
                rs485Set.send485() #enable rs485 to send data
                if self.crc16(data,0,len(data))==0x0000:
                    if data[1]==0x10 : #function code check
                        '''
                        function code:16(0x10). 
                        Address: 100 --totalize, 200--batch
                        '''
                        typeF=int(data[3])
                        if typeF==100 or typeF==200:
                            '''
                            function code:16(0X10)
                            Address:100
                            data:speed
                            '''
                            self.serial.rs485_rep_10(data)
                            if self.prepareData485_rtu(data):
                                self.itemAdd()
                                time.sleep(0.5)
                                self.speed_Change(self.rs232_Speed)
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None
                                    print('Start to count...')
                            rs485Set.read485()
                        elif typeF==10: #valve command
                            '''
                            function code:16(0x10)
                            Address=10
                            No of register:1
                            data:1--close, 2--open
                            '''
                            self.serial.rs485_rep_10(data)
                            if self.cMode !=count_mode[2]:
                                if data[8]==1: #cloase valve
                                    valve.Closevalve()
                                elif data[8]==2:
                                    valve.Openvalve()
                            else:
                                #Data is not support
                                dErr=data[1] | 0x80
                                self.serial.rs485_process_error(dErr,0x03)
                        elif typeF==12:# speed command
                            '''
                            function code:16(0x10)
                            Address=12
                            no of register:1
                            data:speed
                            '''
                            self.serial.rs485_rep_10(data)
                            speed=int(data[8])
                            self.rs232_Speed=str(speed)
                            self.speed_Change(self.rs232_Speed)
                            self.rs232_Command=''
                        elif typeF==15:#dump command
                            '''
                            function code:16(0x10)
                            Address=15
                            data:dump speed
                            '''
                            self.serial.rs485_rep_10(data)
                            self.cMode = count_mode[4]  #set dumpping count mode 
                            self.tempSpeed=self.rs232_Speed 
                            if self.lab:
                                self.lab.config(text='Dumpping...')
                            else:
                                self.commLabel('Dumpping...')
                    
                             #turntable start to rotating 
                        
                            strSpeed=f'Speed: \n200'
                            self.rsSpeedLbl.config(text=strSpeed)
                            self.rs232_Command=""
                        elif typeF==20: #start command
                            '''
                            function code:16(0x10)
                            Address=20 (0x14)
                            data:start
                            '''
                            if self.cMode==count_mode[1] :
                                self.serial.rs485_rep_10(data)
                                self.rs232_Command='Start'
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None
                            elif self.cMode==count_mode[0] and (self.tempMode==count_mode[2] or self.tempMode==count_mode[1]):
                                self.itemAdd()
                                self.speed_Change(self.rs232_Speed)
                                self.serial.rs485_rep_10(data)
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None
                            else:
                                dErr=data[1] | 0x80
                                self.serial.rs485_process_error(dErr,0x02)
                        elif typeF==25: #stop command
                            '''
                            function code:16(0x10)
                            Address=25 (0x19)
                            data:0,speed
                            '''
                            if self.cMode==count_mode[1]:
                                self.serial.rs485_rep_10(data)
                                self.rs232_Command='Stop'
                                
                            elif  self.cMode==count_mode[0]:
                                self.serial.rs485_rep_10(data)
                                turnTable.stop_Turntab()
                            else:
                                dErr=data[1] | 0x80
                                self.serial.rs485_process_error(dErr,0x02)
        
                        elif typeF==30: #zero command
                            '''
                            function code:16(0x10)
                            Address=30 (0x1e)
                            data:0,speed
                            '''
                            if self.cMode==count_mode[1]:
                                self.serial.rs485_rep_10(data)
                                self.rs232_Command='Zero'
                            elif self.cMode==count_mode[0]:
                                self.serial.rs485_rep_10(data)
                                count.clearCount() 
                                self.set_countText(0)   
                            else:
                                dErr=data[1] | 0x80
                                self.serial.rs485_process_error(dErr,0x02)
                        
                        elif typeF==35: #Reset command
                            '''
                            function code:16(0x10)
                            Address=35 (0x23)
                            data:0,reset
                            '''
                            if self.cMode==count_mode[1] or self.cMode==count_mode[2]:
                                self.serial.rs485_rep_10(data)
                                self.rs232_Command='Reset'
                                if self.lab:
                                    self.lab.destroy()
                                    self.lab=None
                            else:
                                dErr=data[1] | 0x80
                                self.serial.rs485_process_error(dErr,0x02)
  
                        else:
                            #data address is not available
                            dErr=data[1] | 0x80
                            self.serial.rs485_process_error(dErr,0x02)
                        
                    elif data[1]==0x04: #function code:read hold register for seed
                        '''
                        function code:0x04(4), read hold register
                        Address=50: retract seed number
                        Adress=60:retract the counter state
                        '''
                        data1=[]
                        data1.append(data[0])
                        data1.append(data[1])
                        if data[3]==50: #retract seed number 
                            t=self.counter_Spe[0] #seed number
                            if t>65535:
                                data1.append(4)#2bytes
                                t1=divmod(t,65536)
                                if t1[0]>255:
                                    t2=divmod(t1[0],256)
                                    data1.append(t2[0])
                                    data1.append(t2[1])
                                    if t1[1]>255:
                                        t2=divmod(t1[0],256)
                                        data1.append(t2[0])
                                        data1.append(t2[1])
                                    else:
                                        data1.append(0)
                                        data1.append(t1[1])
                                else:
                                    data1.append(0)
                                    data1.append(t1[1])
                                    if t1[1]>255:
                                        t2=divmod(t1[0],256)
                                        data1.append(t2[0])
                                        data1.append(t2[1])
                                    else:
                                        data1.append(0)
                                        data1.append(t1[1])
                            else:
                                data1.append(2)#2bytes
                                t1=divmod(t,256)
                                data1.append(t1[0])
                                data1.append(t1[1])
                        elif data[3]==60:#retract counter state
                            t=self.counter_Spe[1] #counter state
                            data1.append(2)#2bytes
                            data1.append(0)
                            data1.append(t)
                        self.serial.rs485_rep_04(data1)
                    else: #invalid function code
                        dErr=data[1] | 0x80
                        self.serial.rs485_process_error(dErr,0x01)
                else:
                    self.lab.config(text='CRC error,send again!')
                    time.sleep(0.01)
                    self.serial.crcCheck_error_respond(data)
                    self.lab.config(text='Waiting for command...!')
            rs485Set.read485()
              
            time.sleep(0.5)

    def itemAdd(self):
        no=obj_batchInf.batch_noGet()
        pack_No=obj_batchInf.no_perbatchGet() #retrieve packet Number
        No_perPack=obj_batchInf.no_perpacketGet()  #retrieve kernel amount per packe
        if pack_No[0]==0:
            self.cMode=count_mode[1] # set to totalize mode
            if self.itemlist<9:
                str1='S000'+str(self.itemlist+1)
            elif self.itemlist<99:
                str1='S00'+str(self.itemlist+1)
            else:
                str1='S0'+str(self.itemlist+1)

            Source_Id=str1
            p=self.itemTv_Rs.insert("", index='end',iid=str(self.itemlist),
                                    text=str(self.itemlist+1),
                                    values=(Source_Id,self.Crop,str(pack_No[0]),str(No_perPack[0]),'',''),
                                    tags='bg'
                                    )
        
                
            self.itemTv_id.append(p)
            self.itemTv_Rs.see(str(self.itemlist))
            self.itemTv_Rs.selection_set(str(self.itemlist))
            count.setCount()
            self.itemlist +=1
          
        else:
            x=0
            for x in range(no):
                if x<9:
                    Source_id='S000'+str(self.itemlist+x+1)
                elif x<99:
                    Source_id='S00'+str(self.itemlist+x+1)
                else:
                    Source_id='S0'+str(self.itemlist+x+1)
                id=self.itemTv_Rs.insert("",'end',iid=str(self.itemlist+x),
                                        text=str(self.itemlist+x+1), 
                                        values=(Source_id,self.Crop,str(pack_No[x]),str(No_perPack[x]), " "," "),
                                        tags='bg'
                                        )
               
                self.itemTv_id.append(id)
            self.itemlist +=no
            #ensure new entry are presented
            self.itemTv_Rs.see(str(self.itemlist-1))
            self.itemTv_Rs.selection_set(str(self.itemlist-no))

            self.setTreeviewNo(no,1)
            if self.rs232_Command !='Recycle' or self.rs232_Command !='recycle':
                if self.tempMode !=count_mode[2]:
                    No=obj_batchInf.no_perpacketGet()
                    count.setCount(1,No[0])
                else:
                    No=obj_batchInf.no_perpacketGet()
                    count.setKernel_batch(No[0])
                
            self.tempMode=count_mode[0]
            self.cMode=count_mode[2] # set to batch mode

        self.update_Treetop()

    #define rs232 count _thread
    def rscount_Thread(self,value):
        global threadExit_Flag
        threadExit_Flag=True
        t_Type='0'
        count.clearCount()
             
        oldNum=0
        newNum=0
        oldNum_0=0

        iChamber =0 # 0--bottom chamber is empty, 1-- bottom chamber is full
        lBatch_No =1   #local variable
        lPackage_No=1  #local variable
        iFinish = 0     #local flag, 0--not finish, 1--batch finished
        iSpeed_Change =False  # True--speed has been changed from H to Low,False--speed is at highspeed
        iRead_flag=False
        speed_stop='0'    #the speed before stop
        EnvelopFlag=False #only totalize use
        No_batch=0
        No_packet=[]
        No_Kernel=[] 
    
        envelopeShow_flip=0 #not carousel mode 0--edge up, 1--edge down; under carousel mode 0--edge down, 1--edge up

        count.countDetect()     #start to count
        logging.info("main:Thread start to run  !")

        while threadExit_Flag:
            if self.cMode==count_mode[1]:  #totalize 
                newNum = count.readCount()
                '''
                if envelope.getEnvelope_state() ==1:
                    if newNum>0 and turnTable.getState():
                        self.rs232_Command='Reset'
                        EnvelopFlag=True
                '''
                if newNum  != oldNum:
                    oldNum=newNum
                    self.set_countText(oldNum)
                    if self.serialType=='rs232':
                        self.sendData(oldNum,'0')
                    else:
                        self.counter_Spe[0]=oldNum
                '''   
                if self.rs232_Command =='M':
                    if rs232_carousel =='0':
                        self.speed_Change(int(self.rs232_Speed))
                        self.rs232_Command=""
                '''
                if self.rs232_Command == 'Start': #for restatr
                    strSpeed=f'Speed: \n {self.rs232_Speed}'
                    self.rsSpeedLbl.config(text=strSpeed)
                    turnTable.start_Turntab(self.rs232_Speed)
                    self.rs232_Command=""
                    if self.lab:
                        self.lab.destroy()  #clear sample alert
                        self.lab=None
                elif self.rs232_Command =='Stop':
                    strSpeed=f'Speed: \n 0'
                    self.rsSpeedLbl.config(text=strSpeed)
                    turnTable.stop_Turntab()
                    self.rs232_Command=""
                    time.sleep(0.2)
                    if self.serialType=='rs232':
                        self.sendData(newNum,'0')
                    self.commLabel('Wait for command...!')

                elif self.rs232_Command =='Zero' :
                    self.set_countText(0)
                    count.clearCount()
                    if self.serialType=='rs232':
                        self.sendData(0,'0')
                    elif self.serialType=='rs485tcp':
                        self.rs485tcp.sendData(50,[0])
                    else:
                        self.counter_Spe[0]=0
                    newNum=0;
                    self.rs232_Command=""

                elif self.rs232_Command == 'Reset':
                    #stop turntable  & clear counter number 
                    turnTable.stop_Turntab()
                    self.tempMode=self.cMode
                    #thread is at idle state
                    self.cMode = count_mode[0]  #set idle count mode 

                    #grab values in the item
                    logging.info("Main:get value %s",self.itemTv_id[self.itemlist-1])
                    #set new value to Amount
                    values_1=self.itemTv_Rs.item(self.itemTv_id[self.itemlist-1],"values")
                    d=datetime.now()
        
                    dt=d.strftime("%d/%m, %H:%M") #get fromat date
                    self.itemTv_Rs.item(self.itemTv_id[self.itemlist-1],
                                            values=(values_1[0],
                                                    values_1[1],
                                                    values_1[2],
                                                    values_1[3],
                                                    str(self.countText.get()),
                                                    dt),
                                                    tags='bg-2')
         
                    #unselect 
                    self.itemTv_Rs.selection_remove(self.itemTv_Rs.selection()[0])
                    if self.serialType=='rs232':
                        #get new data
                        newNum = count.readCount()
                        oldNum=newNum
                        self.set_countText(oldNum)
                        self.sendData(newNum,'0')
                        self.commLabel('Waiting for Data...') 
                        if EnvelopFlag:
                            self.sendComm('Reset')
                            EnvelopFlag=False
                    elif self.serialType=='rs485tcp':
                        self.rs485tcp.sendData(50,[newNum])
                        self.commLabel('Waiting for command...') 
                        self.counter_Spe[1]=0 #counter is in idle
                        self.rs485tcp.sendData(60,[0])
                    else:
                        self.commLabel('Waiting for command...') 
                        self.counter_Spe[0]=self.countText.get() # get kernel number
                        self.counter_Spe[1]=0 #counter is in idle

                    count.clearCount()
                    self.countText.set(0)
                    newNum=0
                    oldNum=0
                    self.rs232_Command=''
                    time.sleep(0.2)    

            elif self.cMode==count_mode[2] :  #batch mode
                 #Obtain batch count information
                if iRead_flag==False:
                    No_batch=obj_batchInf.batch_noGet()    #batct order
                    No_packet = obj_batchInf.no_perbatchGet()  #packet number
                    No_Kernel = obj_batchInf.no_perpacketGet() #kernel perpacket
                    iRead_flag=True
                    self.setBatch_Speed(self.rs232_Speed)
                    #set batch label
                    strBa=f'BatchNo:\n1'
                    self.rsBatchLbl.config(text=strBa)
                    #set package label
                    pNo=str(No_packet[0])+'/'+'1'
                    strPa=f'Packet: \n{pNo}'
                    self.rsPacketLbl.config(text=strPa)

                newNum = count.readCount()
                if self.rs232_Command =='Reset' or self.rs232_Command =='reset':
                    #stop turntable  & clear counter number 
                    if rs232_carousel =='0':
                        turnTable.stop_Turntab()
                        if self.lab:
                            self.lab.destroy()  #clear sample alert
                            self.lab=None
                            
                        self.tempMode=self.cMode
                        #thread is at idle state
                        self.cMode = count_mode[0]  #set idle count mode 

                        #grab values in the item
                        No=self.itemlist-No_batch+lBatch_No-1
                        logging.info("Main:get value %s",self.itemTv_id[No])
                        #set incomplete mark
                        values_1=self.itemTv_Rs.item(self.itemTv_id[No],"values")
                        d=datetime.now()
                        dt=d.strftime("%d/%m, %H:%M") #get fromat date
                        self.itemTv_Rs.item(self.itemTv_id[No],
                                                values=(values_1[0],
                                                        values_1[1],
                                                        values_1[2],
                                                        (values_1[3]+'/'+str(self.countText.get())),
                                                        'partial',
                                                        dt),
                                                        tags='bg-2')
                        lBatch_No +=1
                        while(No_batch>=lBatch_No):
                            No +=1
                            values_1=self.itemTv_Rs.item(self.itemTv_id[No],"values")
                            self.itemTv_Rs.item(self.itemTv_id[No],
                                                values=(values_1[0],
                                                        values_1[1],
                                                        values_1[2],
                                                        (values_1[3]+'/'+'0'),
                                                        'null',
                                                        ''),
                                                        tags='bg-2')
                            lBatch_No +=1
                        lBatch_No -=1
         
                        #unselect 
                        self.itemTv_Rs.selection_remove(self.itemTv_Rs.selection()[0])

                        self.commLabel('Waiting for Data...') 
                        
                        count.clearCount()
                        self.countText.set(0)
                        time.sleep(0.2)    
                        self.rs232_Command="" 
                        newNum=0
                        oldNum=0
                        if valve.state_valve()==1:
                            valve.Openvalve()
                            
                        if self.serialType =='rs485tcp':
                            self.rs485tcp.sendData(60,[0])
                            self.counter_Spe[1]=0 #idle state
                        elif self.serialType =='rs485rtu':
                            self.counter_Spe[1]=0 #idle state
                    else:
                        t=turnTable.getSpeed_Turntab()
                        if t !=0:
                            self.speed_Change('0')
                            speed_stop =t
                            #time.sleep(0.25)
                            if self.serialType=='rs232':
                                self.sendData(newNum,'0')
                            if self.lab ==None:
                                self.commLabel('Pause! Wait command!')
                        self.rs232_Command =='Stop'

                elif self.rs232_Command =='Zero' :
                    newNum=0
                    oldNum=0
                    count.clearCount()
                    self.set_countText(0)
                    self.rs232_Command=""     
                elif self.rs232_Command =='Stop' or self.rs232_Command =='stop':
                    t=turnTable.getSpeed_Turntab()
                    if t !=0:
                        self.speed_Change('0')
                        speed_stop =t
                        #time.sleep(0.25)
                        if self.serialType=='rs232':
                            self.sendData(newNum,'0')
                        if self.lab ==None:
                            self.commLabel('Pause! Wait command!')
                elif self.rs232_Command=='Start' or self.rs232_Command=='start' :
                    if speed_stop=='0':
                        self.speed_Change(self.rs232_Speed)
                    else:
                        self.speed_Change(speed_stop) 
                    if self.lab:
                        self.lab.destroy()  #clear sample alert
                        self.lab=None
                    self.rs232_Command=''
                    
                if (iChamber==0) and (valve.state_valve()==0) and iFinish==1 :
                     #unselect 
                    self.itemTv_Rs.selection_remove(self.itemTv_Rs.selection()[0])

                    iFinish=0
                    lPackage_No=1
                    iSpeed_Change=False
                    iRead_flag=False
                    self.cMode=count_mode[0]
                    if self.serialType=='rs232':
                        self.commLabel('Wating for Data...')
                    elif self.serialType=='rs485tcp':
                        self.rs485tcp.sendData(60,[0])
                        self.commLabel('Wating for command...')
                        self.counter_Spe[1]=0
                    else:
                        self.commLabel('Wating for command...')
                        self.counter_Spe[1]=0
                else:
                    if self.cMode==count_mode[2] and self.rs232_Command !='Stop' :
                        if newNum != oldNum:
                            self.set_countText(newNum)
                            if self.serialType=='rs232':
                                self.sendData(newNum,str(lPackage_No))

                            if newNum < oldNum : #one batch has finished
                                oldNum=newNum
                                iChamber = 1    #bottom chamberisDump
                                
                                valve.Closevalve() #close valve
                                #send out Full
                                if self.serialType=='rs232':
                                    self.sendComm('Full')

                                if self.lab:
                                    self.lab.destroy()  #clear sample alert
                                self.commLabel('Sample Ready !')
                            
                                #retrieve data from treeview
                                No=self.itemlist-No_batch+lBatch_No-1
                                values_1=self.itemTv_Rs.item(self.itemTv_id[No],"values")
                                
                                pNo=str(No_packet[lBatch_No-1])+'/'+str(lPackage_No)  

                                #//change list state
                                self.itemTv_Rs.item(self.itemTv_id[No], 
                                                    values=(values_1[0],
                                                    values_1[1],
                                                    pNo,
                                                    values_1[3],
                                                    '')
                                                    )

                                #check package number
                                if lPackage_No ==No_packet[lBatch_No-1]:
                                    #one batch has finished
                                    if self.serialType=='rs232':
                                        self.sendComm('BatchF')
                                    self.itemTv_Rs.item(self.itemTv_id[No], 
                                                        values=(values_1[0],
                                                                values_1[1],
                                                                pNo,
                                                                values_1[3],
                                                                '',
                                                                'complete'),
                                                                tags='bg-2'
                                                        )

                                    if (lBatch_No) ==No_batch:
                                        strSpeed=f'Speed: \n0'
                                        self.rsSpeedLbl.config(text=strSpeed) 
                                        turnTable.stop_Turntab()
                                        self.tempMode=self.cMode
                                        iFinish=1
                                    else :
                                        count.setKernel_batch(No_Kernel[lBatch_No])
                                        lBatch_No +=1
                                        #change batch and package label display
                                        strBa=f'BatchNo:\n{lBatch_No}'
                                        self.rsBatchLbl.config(text=strBa)

                                        pNo=str(No_packet[lBatch_No-1])+'/'+'1'
                                        strPa=f'Packet: \n{pNo}'
                                        self.rsPacketLbl.config(text=strPa)
                                        #back to original speed
                                        self.speed_Change(str(self.h_speed))
                                        iSpeed_Change=False 
                                        self.setTreeviewNo(No_batch,lBatch_No)
                                        self.itemTv_Rs.selection_set(str(self.tNo))
                                        self.itemTv_Rs.see(str(self.tNo))
                                        lPackage_No=1
                                else:
                                    lPackage_No +=1
                                    pNo=str(No_packet[lBatch_No-1])+'/'+str(lPackage_No)  
                                    strPa=f'Packet: \n{pNo}'
                                    self.rsPacketLbl.config(text=strPa)
                                    #back to original speed
                                    self.speed_Change(str(self.h_speed))
                                    iSpeed_Change=False 
                            else:
                                oldNum = newNum
                                if iChamber ==1:
                                    if (newNum > No_Kernel[lBatch_No-1]/3) and (valve.state_valve()==1) and (turnTable.getState()==1) :
                                        #counter stop
                                        speed_stop=turnTable.getSpeed_Turntab()
                                        self.speed_Change('0')
                                    
                                    if envelope.getEnvelope_state(): #envelope present
                                        if envelopeShow_flip==0:
                                            envelopeShow_flip=1
                                            if self.lab:
                                                self.lab.destroy() #clear sample
                                                if self.serialType=='rs232':
                                                    if rs232_carousel =='0':
                                                        self.sendComm('Empty') 
                                                self.lab=None
                                    elif envelope.getEnvelope_state()==0: 
                                        time.sleep(0.3)
                                        if envelopeShow_flip:
                                            envelopeShow_flip=0
                                            iChamber=0

                                else : 
                                    if valve.state_valve() and not envelope.getEnvelope_state():
                                        valve.Openvalve() 
                                        if not turnTable.getState():
                                            if speed_stop=='0':
                                                self.speed_Change(self.rs232_Speed) 
                                            else:
                                                self.speed_Change(speed_stop) 
                                    if iSpeed_Change==False and  not envelope.getEnvelope_state():
                                        if (No_Kernel[lBatch_No-1] >=50 ) and (newNum > No_Kernel[lBatch_No-1]-12):
                                            turnTable.start_Turntab(self.l_speed)
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True
                                        elif (No_Kernel[lBatch_No-1] < 50 ) and (No_Kernel[lBatch_No-1] >= 20) and (newNum > No_Kernel[lBatch_No-1]-8):
                                            turnTable.start_Turntab(self.l_speed)
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True
                                        elif (No_Kernel[lBatch_No-1] < 20 ) and (newNum > No_Kernel[lBatch_No-1]-5):
                                            turnTable.start_Turntab(str(self.l_speed))
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True
                        else:
                            #check envelop
                            if envelope.getEnvelope_state():
                                if not envelopeShow_flip:
                                    envelopeShow_flip=1
                                    if self.lab:
                                        self.lab.destroy()  #clear sample alert
                                        if self.serialType=='rs232':
                                            if rs232_carousel =='0':
                                                self.sendComm('Empty')
                                        self.lab=None
                            
                            elif envelope.getEnvelope_state()==0:
                                time.sleep(0.3)
                                if envelopeShow_flip:
                                    envelopeShow_flip=0 
                                    iChamber=0
                                elif iChamber==0: 
                                    if valve.state_valve():#valve closed
                                        valve.Openvalve()
                                    if iFinish==0 and turnTable.getState()==0:
                                        self.speed_Change(speed_stop) 

            elif self.cMode == count_mode[3]:    #remote mode
                time.sleep(1)
            elif self.cMode == count_mode[4]:   #dump
                lastCount=newNum
                bDump=1
                iFlag=0
                
                count.setCount() 
                turnTable.start_Turntab(200)
                #set totalize count type
                countType=count.countType
                count.countType=0
                #disable Cancel button
                self.backBtn.config(state='disabled')
                while bDump:
                    if envelope.getEnvelope_state()==0 and valve.state_valve()==1 :
                        valve.Openvalve()

                    newNum = count.readCount()
                    if newNum !=oldNum :
                        oldNum=newNum
                        self.countText.set(oldNum+lastCount)
                        if self.serialType=='rs232':
                            self.sendData(oldNum+lastCount,'0')
                        iFlag=0
                    else :
                        if not iFlag:
                            dt1=time.time()
                            iFlag=1
                        else :
                            dt2=time.time()
                            if (dt2 - dt1)>4 : #3 secons
                                time.sleep(0.2)
                                if self.serialType=='rs232':
                                    self.sendData(newNum+lastCount,'0')
                                elif self.serialType=='rs485tcp':
                                    self.rs485tcp.sendData(60,[0])
                                    da=self.countText.get()
                                    self.rs485tcp.sendData(50,[da])
                                else:
                                    self.counter_Spe[0]=self.countText.get()
                                valve.Closevalve()
                                time.sleep(1)
                                valve.Openvalve()
                                turnTable.stop_Turntab()
                                #dump finished
                                if self.serialType=='rs232':
                                    self.sendComm('Dump')
                            
                                bDump=0 #exit dump
                                break
                self.backBtn.config(state='normal')
                count.countType=countType
                if self.tempMode==count_mode[2] :
                    #set new value to remainder
                    d2 =datetime.now()
                    str_d2=d2.strftime('%d/%m,%H:%M')
                    values_1=self.itemTv_Rs.item(self.itemTv_id[(self.itemlist-No_batch)+lBatch_No-1],"values")
                    self.itemTv_Rs.item(self.itemTv_id[(self.itemlist-No_batch)+lBatch_No-1],
                                        values=(values_1[0],
                                                values_1[1],
                                                values_1[2],
                                                values_1[3],
                                                str(self.countText.get()),
                                                str_d2),
                                                tags='bg-2')
                    iRead_flag=False
                    self.tempMode=count_mode[0]
                if self.lab:
                    if self.serialType=='rs232':
                        self.lab.config(text='Wating for Data...')
                    elif self.serialType=='rs485tcp':
                        self.lab.config(text='Wating for command...')
                        self.rs485tcp.sendData(60,[0])

                    else:
                        self.lab.config(text='Wating for command...')
                        self.counter_Spe[1]=0
                        
                else:   
                    if self.serialType=='rs232': 
                        self.commLabel('Wating for Data...') 
                    elif self.serialType=='rs485tcp':
                        self.commLabel('Wating for command...')
                        self.rs485tcp.sendData(60,[0])
                    else:
                        self.commLabel('Wating for command...')
                        self.counter_Spe[1]=0
                count.clearCount()
                newNum=0
                oldNum=0  
                self.cMode=count_mode[0]
                
            else:
                newNum= count.readCount()
                if newNum  != oldNum_0:
                    oldNum_0=newNum
                    self.set_countText(oldNum_0)
                    self.counter_Spe[0]=oldNum_0
   
                iChamber=0
                lBatch_No=1
                lPackage_No=1
                iFinish=0
                iSpeed_Change=False
                iRead_flag=False
        
            time.sleep(0.01)
        logging.info("Thread exit counting !")
    def data_composite(self,intVal,packetVal):
         byteValue=chr(2)+chr(100)+chr(31)+str(intVal)+chr(31)+packetVal+chr(31)+chr(3)
         return byteValue

    def data_respond(self,strVal):
         byteValue=chr(2)+chr(99)+chr(31)+strVal+chr(31)+chr(3)
         return byteValue

    def sendComm(self, strVal):
         byteVal=self.data_respond(strVal)
         time.sleep(0.25)
         self.serial.sendData(byteVal)

    def sendData(self,intVal,packetVal):
         byteVal=self.data_composite(intVal,packetVal)
         self.serial.sendData(byteVal)    

    def commLabel(self,strV):
         self.lab=TK.Label(self.root,text=strV,
                        bg='lightblue',
                        fg='darkred',
                        font=('Times',30))
         self.lab.place(x=560,y=82,height=60,width=400)

    '''
     implementation of CRC-16/CCITT-FALSE
        data : bytearray of the data you want to calculate CRC for
        offset : from which offset you want to start calculating CRC
        length : to which offset you want to calculate CRC
    '''
    def crc16(self,data : bytearray, offset, length):
         if data is None or offset < 0 or offset > len(data) - 1 and offset + length > len(data):
                return -1 #false
         crc = 0xFFFF
         for i in range(length):
             crc ^= data[offset + i]
             for j in range(8):
                 print(crc)
                 if ((crc & 0x1) == 1):
                    print("bb1=", crc)
                    crc = int((crc / 2)) ^ 40961
                    print("bb2=", crc)
                 else:
                    crc = int(crc / 2)
         return crc & 0xFFFF

    def setBatch_Speed(self,speedVal):
         self.h_speed=int(speedVal)
         speed_stop=speedVal
         if self.h_speed <=10:
            self.l_speed=10
         elif self.h_speed <20 :
            self.l_speed=10
         elif self.h_speed>100:
            self.l_speed=50
         else:
            self.l_speed=int(self.h_speed/2)

    def speed_Change(self,intVal):#intVal data type is string
        strVal=intVal
        strSpeed=f'Speed: \n{strVal}'
        self.rsSpeedLbl.config(text=strSpeed)
        turnTable.start_Turntab(strVal)

    def cancel_click(self):
         global page_Back,threadExit_Flag

         page_Back ="back"
         self.listen_Flag=False
         if self.serialType=='rs232':
            self.rsThread_02.join()
         elif self.serialType=='rs485rtu':
             self.rsThread_rs485rtu.join()
         else:
             self.rs485_flag=False
             self.rsThread_rs485tcp.join()

         threadExit_Flag =False
         self.rsThread_03.join()
        
         if self.serialType=='rs232' or self.serialType=='rs485rtu':
             self.serial.sendData('Disconnect')
             time.sleep(0.5)
             self.serial.closePort()
         turnTable.stop_Turntab()
         count.clearCount()
         count.cleanDetect()

         logging.info("Main: back to main page")
         self.powerThread.join()
         page_Back= ''
         self.root.destroy()


"""
    define independent mode
"""
class mainPage_020(frlsWin):
    def __init__(self,_strVal):
        super().__init__()

        self.doneBtn_flag=False
        self.countType=_strVal # '2'--independent, '4'--load file
        #change title of window
        self.label['text']="Indepdent Mode"
        self.style=TTK.Style()
         
        #Counter start to counter by create a new thread
        self.itemTv_id=[] #empty list
        self.itemlist=0   #list number
        self.treeTop=0  #initial position
        self.tNo=0      #current count number
    
        self.lineNo=[]  #current line number in the list of itemlist for loadfile
        self.start_row=0  #new file start row
        self.T_batchNo=0 #finished batch number
        self.tempMode=count_mode[0]

        self.bSave_flag=False # True--file have been saved, 
    
        self.x0=0
        self.y0=0
        self.xm=0
        self.ym=0
        self.xr=0
        self.yr=0

        self.speedText='50'
        #Create a common frame
        self.commFr=TK.Frame(self.root,bg='gray')
        self.commFr.pack(fill=TK.X)
        
        #create a back button       
        self.back_Btn =TK.Button(self.commFr,text="Home",
                                  bg="light slate gray", relief=TK.RAISED,
                                  font=Font(family="Times Newn Roman",size=24),
                                  command=self.back_click)
        self.back_Btn.pack(side=TK.LEFT,padx=10,ipadx=10,ipady=10)
        if self.countType=='2': #independent mode
            #create checkoutbutton for totalize and batch counter mode,defualt setting is totalize
            self.myFont=Font(family="Times Newn Roman",size=22)
            #Set check button variable
            self.radio_BT=TK.IntVar()  
            self.radio_BT.set(1)    #initial radio button 
            self.radioBtn_t =TK.Radiobutton(self.commFr, text="Totalize ",
                                            variable=self.radio_BT,
                                            value=1,
                                            bg="light slate gray",
                                            font=self.myFont,
                                            command=self.radio_click)
            self.radioBtn_t.pack(side=TK.LEFT,padx=30,ipadx=10,ipady=10)

            self.radioBtn_b =TK.Radiobutton(self.commFr, text="Batch  ",
                                            variable=self.radio_BT,
                                            value=2,          
                                            bg="light slate gray", 
                                            font=self.myFont,
                                            command=self.radio_click)
            self.radioBtn_b.pack(side=TK.LEFT,padx=20,ipadx=5,ipady=10)
            self.radioBtn_t.select()
            self.radio_state=1 # 1 for totalize, 2 for batch
        elif self.countType[0]=='4': #load file
            #setup font
            self.delimite=';'
            self.myFont=Font(family="Times",size=16)

            self.speedBtn = TK.Button(self.commFr,text=" Speed  \n50",
                                    bg="gray",relief=TK.RAISED,
                                    font=self.myFont,
                                    command=self.speed_click)
            self.speedBtn.pack(side=TK.LEFT,padx=10,ipady=5,ipadx=5)

            #create a batch label
            self.rsBatchLbl=TK.Label(self.commFr,text='BatchNo:\n1',
                                    font=self.myFont,
                                    bg='gray'
                                    )
            self.rsBatchLbl.pack(side=TK.LEFT,padx=35,ipady=5,ipadx=5)
        
            #create a packet label
            self.rsPacketLbl = TK.Label(self.commFr,text='Packet:\n1',
                                        font = self.myFont,
                                        bg='gray'
                                        )
            self.rsPacketLbl.pack(side=TK.LEFT,padx=35,ipady=5,ipadx=5)
            
            #create a load file button
            self.loadfileBtn=TK.Button(self.commFr,text='Load File',
                                        bg='gray',fg='red',
                                        relief='raised',
                                        font=self.myFont,
                                        command=self.open_file)
            self.loadfileBtn.pack(side='left',padx=40,ipady=10,ipadx=5)

        #create a clear Button
        self.countClear = TK.Button(self.commFr, text="Zero",
                                   bg="gray",relief=TK.RAISED,
                                   font=self.myFont,
                                   command=self.countClear_click)
        self.countClear.pack(side=TK.RIGHT,padx=10,pady=1, ipady=10)

        self.countText=TK.IntVar()
        self.countText.set(0)
        self.countLbl_Num = TK.Label(self.commFr,textvariable=self.countText,
                                     bg='dark khaki',font=Font(family='Times New Roman',size=26))
        self.countLbl_Num.pack(side=TK.RIGHT,pady=1,ipady=6)

        #Create counting label
        self.countLbl =TK.Label(self.commFr,text="Count :",
                                bg='khaki',font=self.myFont)
        self.countLbl.pack(side=TK.RIGHT,padx=25,pady=1,ipady=6)
        
        self.clear_listBtn = TK.Button(self.commFr,text='Clear',
                                        bg="gray", relief=TK.RAISED,
                                        font=self.myFont,
                                        command=self.clearlistBtn_click)
        self.clear_listBtn.pack(side=TK.RIGHT,padx=25,pady=1,ipady=10)
        self.commFr.pack(pady=1,padx=20)

        #create a frame for input message
        self.inputMsg = TK.Frame(self.root,bg='gray')

        #create a frame for counting
        self.countFrm = TK.Frame(self.root,bg='gray')
        self.treeview_set()
        if self.countType=='2': #indepedent mode
            #create a crop button and an entry
            self.cropText=TK.StringVar()
            self.cropText.set("  Corn   ")
            self.cropBtn = TK.Button(self.inputMsg, textvariable=self.cropText,
                                    bg="gray",relief=TK.RAISED,
                                    font=self.myFont,
                                    command=self.crop_click)
            self.cropBtn.grid(row=0,column=0,pady=25,ipady=10,sticky=TK.NW)

            #create a labels and an entry
            self.speedBtn = TK.Button(self.inputMsg,text=" Speed  \n50",
                                    bg="gray",relief=TK.RAISED,
                                    font=self.myFont,
                                    command=self.speed_click)
            self.speedBtn.grid(row=1,column=0,pady=20,ipady=10,sticky=TK.NW)

            #create a Dump button
            self.dumpBtn = TK.Button(self.inputMsg,text="Dump",
                                    bg="gray",relief=TK.RAISED,
                                    width=6,
                                    font=self.myFont,
                                    command=self.dumpBtn_click)
            self.dumpBtn.grid(row=2,column=0,pady=150,ipady=5,sticky=TK.NW)
            
            self.inputMsg.place(x=30,y=150)

            self.treeview_totalize()
            self.countFrm.place(x=200,y=150,width=900)

            #create a frame for counting
            self.paraFrm = TK.Frame(self.root,bg='gray')
            #set special font for sensitivity,Horigate & Vertigate
            self.myFont_m=Font(family="Times Newn Roman",size=12,weight="bold") 
            self.sensitivityLb1 = TK.Label(self.paraFrm,text=" Sensitivity \n6",bg='khaki',
                                        font=self.myFont_m)
            self.sensitivityLb1.grid(row=0,column=0,pady=15,ipady=2)
        
            self.horiLb1 = TK.Label(self.paraFrm,text="  HoriGate   \n15",bg='khaki',
                                    font=self.myFont_m)
            self.horiLb1.grid(row=1,column=0,pady=15,ipady=2)
            
            self.vertiLb1 = TK.Label(self.paraFrm,text=" VertiGate  \n15",bg='khaki',
                                    font=self.myFont_m)
            self.vertiLb1.grid(row=2,column=0,pady=15,ipady=2)
            self.paraFrm.place(x=1130,y=150)
            
        elif self.countType[0]=='4': #load file mode
            #create treeviewFrm
            #open excel file & fill treeview
            self.treeview_file()
            self.countFrm.pack(fill=TK.X,pady=10,padx=40)
        
        #create a frame for control buttons
        self.ctlbtnFrm = TK.Frame(self.root,bg='gray')
        
        #hideClear button

        if self.countType[0]=='4':  #load file mode
            #create a dump button 
            self.dumpBtn = TK.Button(self.ctlbtnFrm,text="Dump",
                                    bg="gray",relief=TK.RAISED,
                                    width=8,
                                    font=self.myFont,
                                    command=self.dumpBtn_click)
            self.dumpBtn.pack(side=TK.LEFT,padx=25,pady=11,ipady=5)
            #create a done button
            self.doneBtn = TK.Button(self.ctlbtnFrm,text="Done", state=TK.NORMAL,
                                    bg="gray",relief=TK.RAISED,
                                    width=8,
                                    font=self.myFont,
                                    command=self.doneBtnFl_click)
            self.doneBtn.pack(side=TK.LEFT,padx=25,pady=15,ipady=5)
            self.doneBtn.config(state='disabled')

            #create a save file button
            self.savefileBtn=TK.Button(self.ctlbtnFrm,text='Save',
                                        bg='gray',relief=TK.RAISED,
                                        width=8,
                                        font=self.myFont,
                                        command=self.savefile_click)
            self.savefileBtn.pack(side=TK.LEFT,padx=25,pady=15,ipady=5)
            self.savefileBtn.pack_forget()
            
            #create a barcode input entry
            self.barcode=TK.StringVar()
            self.barcode.set('')
            self.barcode_En=TK.Entry(self.ctlbtnFrm,textvariable=self.barcode,width=30)
            self.barcode_En.pack(side=TK.RIGHT,padx=10,pady=15,ipady=10)
            self.barcode_En.config(state='disabled')
            #create a key-enter bind
            self.barcode_En.bind('<Return>',self.barcode_Entry)
            self.barcode_En.bind('<Double-Button-1>',self.keyBd,add='+')
       
            #create a label for entry
            self.barcode_Ind=TK.Label(self.ctlbtnFrm,text='Barcode:',
                                    bg='gray',
                                    font=self.myFont
                                    )
            self.barcode_Ind.pack(side=TK.RIGHT,pady=10,ipady=5,ipadx=10)
        
            self.ctlbtnFrm.place(x=40,y=630,width=1150)

        self.ctlbtnFrm.place(x=40,y=630)

        #initial cropparameter
        self.obj_Crop=crp.cropPara()
        
        #set turntable
        turnTable.setSpeed_Turntab(self.speedText)
        turnTable.stop_Turntab()

        #set count mode 
        self.cMode = count_mode[0]  #set count mode idle
        
        logging.info("Main : create a thread of getcount_thread")
        self.countThread_01=threading.Thread(target=self.getcount_Thread, name="countThread",args=(1,))
        self.countThread_01.setDaemon(True)
        self.countThread_01.start()
        logging.info("Main : thread has been created")

        #create a save file thread
        logging.info("Main : create a thread of poweroff_save")
        self.poweroffThread_save=threading.Thread(target=self.esc2_turnoff, name="psThread",args=(1,))
        self.poweroffThread_save.setDaemon(True)
        self.poweroffThread_save.start()
        logging.info("Main : thread has been created")

        self.root.mainloop()
    '''
        create a save file thread function at power off
    '''
    def esc2_turnoff(self,intVal):
        global page_Back
        while True:
            if self.bPoweroff_flag:
                if self.itemlist>0:
                    msbox=Message_dialogue(self.root,'Power off\nSave data ?','yes-no','Reminder !')
                    if msbox.iflag=='OK':
                        d_path=self.is_USB()
                        if not d_path:
                            Message_dialogue(self.root,'Please insert USB stick !\n do not close this window\n unitll USB stick is ready for use !','ok','Reminder !')
                            d_path=self.is_USB()
                            if not d_path:
                                return
                            else:
                                if self.countType[0]=='4': #load file mode
                                    if not self.isDone():
                                        self.doneBtnFl_click() 
                                    self.save_file(self.itemTv_Lf,d_path)
                                elif self.radio_state==1:
                                    if not self.isDone():
                                        self.doneBtnTota_click()
                                    self.save_file(self.itemTv_totalize,d_path)
                                else:
                                    if not self.isDone():
                                        self.doneBtnBatch_click()
                                    self.save_file(self.itemTv_batch,d_path)  
                        else:
                            if self.countType[0]=='4': #load file mode
                                if not self.isDone():
                                    self.doneBtnFl_click() 
                                self.save_file(self.itemTv_Lf,d_path)
                            elif self.radio_state==1:
                                if not self.isDone():
                                    self.doneBtnTota_click()
                                self.save_file(self.itemTv_totalize,d_path)
                            else:
                                if not self.isDone():
                                    self.doneBtnBatch_click()
                                self.save_file(self.itemTv_batch,d_path)
                            
                                
                #turn raspberry pi
                self.lblWarning=TK.Label(self.root,text='System is shutdown...',
                                        bg='gray',fg='blue',
                                        relief=TK.RAISED,
                                        font=self.myFont)
                self.lblWarning.place(x=500,y=250)
                time.sleep(3)
                pi_shutdown.TurnOff19()
                os.system('sudo shutdown -h now')
            time.sleep(0.05)
            if page_Back=='back':#exit thread
                break


    def treeview_set(self):
        #create a treeview' style and use theme 'alt'
        self.style.theme_use('alt')
        #set the height of treeview
        self.style.configure("TC.Treeview",rowheight=40,font=('Times',24),background='silver',foreground='black')
        #set heading size
        self.style.configure("TC.Treeview.Heading",font=('Times',18),background='silver',foreground='black')
        #set select color strip
        self.style.map('Treeview',background=[('selected','blue')])

        #create a form for treeview and scrollbar
        self.scrtreeFrm= TK.Frame(self.countFrm,bg='gray')
        self.scrtreeFrm.pack(fill='x')

        #define vertical scrollbar
        self.style.element_create('My.Vertical.TScrollbar.trough','from','clam')
        self.style.element_create('My.Vertical.TScrollbar.thumb','from','clam')
        self.style.element_create('My.Vertical.TScrollbar.grip','from','clam')
        #define scrollbar size
        self.style.configure('My.Vertical.TScrollbar',arrowsize=30)

    def treeview_totalize(self): 
        #create a scrollbar
        self.scrobar_Treeview=TTK.Scrollbar(self.scrtreeFrm,orient='vertical',style='My.Vertical.TScrollbar') 
    
        self.itemTv_totalize = TTK.Treeview(self.scrtreeFrm,
                                         columns=("Source_ID","Count","Comment"),
                                         style='TC.Treeview',
                                         height=11)
        self.scrobar_Treeview.config(command=self.itemTv_totalize.yview)
        self.itemTv_totalize.config(yscrollcommand=self.scrobar_Treeview.set)
        self.scrobar_Treeview.pack(side='right',fill='y')

        #format column
        #elf.itemTv_totalize["column"]=("Source_ID","Count","Comment")
        self.itemTv_totalize.column("#0",width=40,stretch=TK.YES,anchor=TK.W)
        self.itemTv_totalize.column("Source_ID",width=200,stretch=TK.YES,anchor=TK.W)
        self.itemTv_totalize.column("Count",width=80,stretch=TK.YES,anchor=TK.W)
        self.itemTv_totalize.column("Comment",width=150,stretch=TK.YES,anchor=TK.CENTER)

        self.itemTv_totalize.heading('#0',text='No',anchor=TK.W)
        self.itemTv_totalize.heading('Source_ID',text='Source_ID',anchor=TK.W)
        self.itemTv_totalize.heading('Count',text='Count',anchor=TK.W)
        self.itemTv_totalize.heading('Comment',text='Comment',anchor=TK.CENTER)

        self.itemTv_totalize.pack(fill=TK.X)
        
        #config color
        self.itemTv_totalize.tag_configure('bg',background='sky blue')
        self.itemTv_totalize.tag_configure('bg-1',background='steel blue')
        self.itemTv_totalize.tag_configure('bg-2',background='dimgray') #finsihed color
        #config color
        self.itemTv_totalize.tag_configure('bg',background='lightblue')

        self.itemTv_totalize.bind('<Button-1>',self.mouse_Pressed)
        self.itemTv_totalize.bind('<B1-Motion>',self.mouse_Move,add='+')
        self.itemTv_totalize.bind('<ButtonRelease-1>',self.mouse_Release,add='+')

        #create a start button
        self.buttonStart=TK.StringVar()
        self.buttonStart.set("Start")
        self.startBtn = TK.Button(self.countFrm,textvariable=self.buttonStart,
                                  bg="gray",relief=TK.RAISED,
                                  width=8,
                                  font=self.myFont,
                                  command=self.startBtn_click)
        self.startBtn.pack(side=TK.LEFT,pady=15,ipady=5)

        #create a done button
        self.doneBtn = TK.Button(self.countFrm,text="Done", state=TK.NORMAL,
                                    bg="gray",relief=TK.RAISED,
                                    width=8,
                                    font=self.myFont,
                                    command=self.doneBtnTota_click)
        self.doneBtn.pack(side=TK.LEFT,padx=50,pady=15,ipady=5)
        self.doneBtn.config(state='disabled')
        
        self.buttonValve=TK.StringVar()
        self.buttonValve.set("Close valve")
        self.valveBtn = TK.Button(self.countFrm,textvariable=self.buttonValve,
                                  bg="gray",relief=TK.RAISED,
                                  width=8,
                                  font=self.myFont,
                                  command=self.valveBtn_click)
        self.valveBtn.pack(side=TK.RIGHT,padx=25,pady=15,ipadx=6,ipady=5)

        #set count mode 
        self.cMode = count_mode[1]  #set totalize count mode

        if valve.state_valve() :
            valve.Openvalve()
        
        turnTable.setSpeed_Turntab(self.speedText)
        turnTable.stop_Turntab()
         #set count to totalize mode

        count.setCount(0) #set totalize

    def treeview_batch(self):
        self.scrobar_Treeview=TTK.Scrollbar(self.scrtreeFrm,orient='vertical',style='My.Vertical.TScrollbar') 
        self.itemTv_batch = TTK.Treeview(self.scrtreeFrm,
                                         columns=("Source_ID","Packet","No/Packetspd","Remainder","Comment"),
                                         style='TC.Treeview',
                                         height=11)

        self.scrobar_Treeview.config(command=self.itemTv_batch.yview)
        self.itemTv_batch.config(yscrollcommand=self.scrobar_Treeview.set)
        self.scrobar_Treeview.pack(side='right',fill='y')
      
        #set column format
        self.itemTv_batch.column("#0",width=60,stretch=TK.NO,anchor=TK.W)
        self.itemTv_batch.column("Source_ID",width=210,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_batch.column("Packet",width=110,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_batch.column("No/Packetspd",width=210,stretch=TK.YES,anchor=TK.CENTER)
        self.itemTv_batch.column("Remainder",width=130,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_batch.column("Comment",width=200,stretch=TK.NO,anchor=TK.CENTER)
        #define headline
        self.itemTv_batch.heading("#0",text="No",anchor=TK.W)
        self.itemTv_batch.heading("Source_ID",text="Source_ID",anchor=TK.CENTER)
        self.itemTv_batch.heading("Packet",text="Packet",anchor=TK.CENTER)
        self.itemTv_batch.heading("No/Packetspd",text="No/Packet: Speed",anchor=TK.CENTER)
        self.itemTv_batch.heading("Remainder",text="Remainder",anchor=TK.CENTER)
        self.itemTv_batch.heading("Comment",text="Comment",anchor=TK.CENTER)
        self.itemTv_batch.pack(fill=TK.X)

        #config color
        self.itemTv_batch.tag_configure('bg',background='sky blue')
        self.itemTv_batch.tag_configure('bg-1',background='steel blue')
        self.itemTv_batch.tag_configure('bg-2',background='dimgray') #finsihed color

        self.itemTv_batch.bind('<Button-1>',self.mouse_Pressed)
        self.itemTv_batch.bind('<B1-Motion>',self.mouse_Move,add='+')
        self.itemTv_batch.bind('<ButtonRelease-1>',self.mouse_Release,add='+')

        #create a next button
        self.buttonStart=TK.StringVar()
        self.buttonStart.set("Next")
        self.startBtn = TK.Button(self.countFrm,textvariable=self.buttonStart,
                                  width=8,
                                  bg="gray",relief=TK.RAISED,
                                  font=self.myFont,
                                  state='disabled',
                                  command=self.startBtn_click
                                  )
        self.startBtn.pack(side=TK.LEFT,padx=30,pady=15,ipadx=5,ipady=5)
        #create a done button
        self.doneBtn = TK.Button(self.countFrm,text="Done", state=TK.NORMAL,
                                 width=8,
                                 bg="gray",relief=TK.RAISED,
                                 font=self.myFont,
                                 command=self.doneBtnBatch_click
                                 )
        self.doneBtn.pack(side=TK.LEFT,padx=30,pady=15,ipadx=5,ipady=5)

        #set counter low speed
        #self.setBatch_Speed(self.speedText)

     
     #create tree_view backbone   
    def treeview_file(self):
        self.scrobar_Treeview=TTK.Scrollbar(self.scrtreeFrm,orient='vertical',style='My.Vertical.TScrollbar') 
        
        #create a treeview for batch count
        self.itemTv_Lf = TTK.Treeview(self.scrtreeFrm,
                                        columns=("Source_ID","Packet_ID","Packet","No/Packet","Location","Remainder","Comment"),
                                        style='TC.Treeview', 
                                        height=11)

        self.scrobar_Treeview.config(command=self.itemTv_Lf.yview)
        self.itemTv_Lf.config(yscrollcommand=self.scrobar_Treeview.set)
        self.scrobar_Treeview.pack(side='right',fill='y')  

        #config color
        self.itemTv_Lf.tag_configure('bg',background='sky blue')
        self.itemTv_Lf.tag_configure('bg-1',background='steel blue')
        self.itemTv_Lf.tag_configure('bg-2',background='dimgray') #finsihed color

        self.itemTv_Lf.heading("#0",text="No",anchor=TK.W)
        self.itemTv_Lf.heading("Source_ID",text="Source_ID",anchor=TK.CENTER)
        self.itemTv_Lf.heading("Packet_ID",text="Packet_ID",anchor=TK.CENTER)
        self.itemTv_Lf.heading("Packet",text="Packet",anchor=TK.CENTER)
        self.itemTv_Lf.heading("No/Packet",text="No/Packet",anchor=TK.CENTER)
        self.itemTv_Lf.heading("Location",text="Location",anchor=TK.CENTER)
        self.itemTv_Lf.heading("Remainder",text="Remainder",anchor=TK.CENTER)
        self.itemTv_Lf.heading("Comment",text="Comment",anchor=TK.CENTER) 
        #self.itemTv_Rs.config(style='TC.Treeview.heading')
        #format column
        self.itemTv_Lf.column("#0",width=80,stretch=TK.NO,anchor=TK.W)
        self.itemTv_Lf.column("Source_ID",width=235,stretch=TK.NO,anchor=TK.E)
        self.itemTv_Lf.column("Packet_ID",width=170,stretch=TK.NO,anchor=TK.E)
        self.itemTv_Lf.column("Packet",width=100,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Lf.column("No/Packet",width=150,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Lf.column("Location",width=100,stretch=TK.NO,anchor=TK.E)
        self.itemTv_Lf.column("Remainder",width=130,stretch=TK.NO,anchor=TK.CENTER)
        self.itemTv_Lf.column("Comment",width=220,stretch=TK.NO,anchor=TK.CENTER) 

        self.itemTv_Lf.bind('<Button-1>',self.mouse_Pressed)
        self.itemTv_Lf.bind('<B1-Motion>',self.mouse_Move,add='+')
        self.itemTv_Lf.bind('<ButtonRelease-1>',self.mouse_Release,add='+')
        self.itemTv_Lf.pack(fill=TK.X)

     #open excel file
    def open_file(self):
        iFlag=True  
        #open filedialog to gate file name
        if self.is_USB():
            openFile=File_dialogue(self.root,'Open file')
            if openFile.iflag=='OK':
                fileName=openFile.current_dir
                del openFile
                if fileName=='':
                    return
            else:
                del openFile
                return
        else:
            Message_dialogue(self.root,'Please insert USB stick !','ok','Reminder !')
            return 
        try:
            row_colItem=[['0' for col in range(8)]for row in range(30000)] #define maximum row 30000lines
            Ws=load_workbook(fileName)
            Sht_1=Ws.active

            #for row in range(1,9,1):
            row=1
            bFlag=False
            self.start_row=self.itemlist 
            while True:
                for col in range(1,8,1):
                    c_char=get_column_letter(col)
                    cell_char=Sht_1[c_char+str(row+1)].value
                    if cell_char :
                        if type(cell_char) is float:
                            celStr=str(int(cell_char))
                            row_colItem[row-1][col-1]=celStr
                        elif type(cell_char)is int:
                            celStr=str(cell_char)
                            row_colItem[row-1][col-1]=celStr
                        else:
                            index_po=cell_char.find('.')
                            if index_po ==-1:
                                row_colItem[row-1][col-1]=cell_char
                            else:
                                celStr=cell_char[:index_po]
                                row_colItem[row-1][col-1]=celStr
                    else:
                        if col==1:
                            bFlag=True
                            break
                if not bFlag:
                    p=self.itemTv_Lf.insert("",'end',iid=str(self.itemlist),
                                            text=str(self.itemlist+1),
                                            values=row_colItem[row-1],
                                            tags='bg'
                                            )
                    self.itemTv_id.append(p)
                    self.itemlist +=1
                    row +=1
                else :
                    break

            self.T_batchNo=row-1      
            self.bSave_flag=False
            self.savefileBtn.pack_forget()
            row_colItem.clear()
            self.barcode_En.config(state='normal')
            self.loadfileBtn.pack_forget()
            self.clear_listBtn.config(state='normal')
            self.barcode.set('')
            self.barcode_En.focus()

        except Exception:
            Message_dialogue(self.root,'Open file failed !','ok','Reminder !')
            iFlag=False
        finally:
            return iFlag
     
     #handle entry barcode
    def barcode_Entry(self,event):
         #extract souce_Id & package Id
         enVarible=self.barcode.get() 
         if enVarible !='':
            if self.countType=='4-y':
                index=enVarible.find(self.delimite)
                if index>0:
                    var=enVarible.split(self.delimite)
                    self.packet_Id=var[0]
                    self.source_Id=var[1]
                    #search id in the list
                    if self.tempMode==count_mode[0]:
                        self.lineNo.clear() #remove data
                        mesg=self.sourceID_search()
                        if mesg !=True:
                            Message_dialogue(self.root,mesg,'ok','Reminder !')
                            self.barcode.set('')
                            self.barcode_En.focus()
                            return False
                    mesg=self.packetID_search()
                    if mesg !=True:
                        Message_dialogue(self.root,mesg,'ok','Reminder !')
                        self.barcode.set('')
                        self.barcode_En.focus()
                        return False
                    self.barcode_En.config(state='disabled')
                    batch_No=obj_batchInf.batch_noGet()
                    No=obj_batchInf.no_perpacketGet()   
                    cBatch_No=self.get_cBatch_No()
                    cBatch_No +=1
                    bStr=str(cBatch_No)+"/"+str(batch_No)
                    if No[0]==0: #totalize
                        bStr='1'
                    self.rsBatchLbl['text']=f'BatchNo:\n{bStr}'
                    temSpeed=str(self.getBatch_speed())
                    self.setBatch_Speed(temSpeed)
                    self.speed_Change(str(self.h_speed))
                    time.sleep(0.5)
                    self.cMode=count_mode[3]

                    self.doneBtn.config(state='normal')
                    self.clear_listBtn.config(state='disabled')
                    return True
                else:
                    self.source_Id=enVarible
                    Message_dialogue(self.root,'delimilte is ;','ok','Reminder !')
                    self.barcode.set('')
                    self.barcode_En.focus()
            else:
                self.lineNo.clear() #remove data
                self.source_Id=enVarible
                mesg=self.sourceID_search()
                if mesg !=True:
                    Message_dialogue(self.root,mesg,'ok','Reminder !')
                    self.barcode.set('')
                    self.barcode_En.focus()
                    return False
                else:
                    self.barcode_En.config(state='disabled')
                    self.tNo=self.lineNo[0] #set current NO
                    No=obj_batchInf.no_perpacketGet()
                    if No[0]==0: #totalize
                        count.clearCount()
                        count.setCount(0,0)
                        
                    else:
                        batch_No=obj_batchInf.batch_noGet()
                        bStr='1'+"/"+str(batch_No)
                        self.rsBatchLbl['text']=f'BatchNo: \n{bStr}'
                        count.setCount(1,No[0])
                       
                    count.lCount=self.countText.get()
                    self.speedText=str(self.getBatch_speed()) 
                    self.setBatch_Speed(self.speedText)
                    
                    '''
                        enter count process, 
                        ensure the item selected is visuable
                    '''
                    self.itemTv_Lf.see(str(self.tNo))
                    self.itemTv_Lf.selection_set(str(self.tNo))
                
                    turnTable.setSpeed_Turntab(self.h_speed)
                    turnTable.start_Turntab()
                    time.sleep(0.5)
                    self.tempMode=count_mode[0]
                    self.cMode=count_mode[3]

                    self.doneBtn.config(state='normal')
                    self.clear_listBtn.config(state='disabled')
                    return True

         else:
             self.barcode_En.focus()
             return False
     
     #bring out keyboard
    def keyBd(self,event):
         if self.itemlist==0:
            Message_dialogue(self.root,'Please load a file','ok','Reminder !')
            return False

         position=[200,250] #keyboard postion
         kb=keyF_keyboard(self.root,position,'Barcode:','')
         #use filedialog to save a file
         fileName=kb.get_Filename()
         self.barcode.set(fileName)
         #create a virtual keybaord
        
         self.barcode_En.focus()
         
         self.barcode_Entry('\n')

    def savefile_click(self):
        d_path=self.is_USB()
        if not d_path:
            Message_dialogue(self.root,'Please insert USB stick','ok','Reminder !')    
        else:
            if self.countType[0]=='4':
                self.save_file(self.itemTv_Lf,d_path)
            elif self.radio_state==1:
                self.save_file(self.itemTv_totalize,d_path)
            else:
                self.save_file(self.itemTv_batch,d_path) 
             
    def save_file(self,object,d_path):
         
        d=datetime.now()
        dt=d.strftime("%b %d, %H:%M") #get fromat date
        #create a workbook
        Ws=Workbook()
        Sht=Ws.active
        Sht.title='Esc2 sample'
        Sht['C1']='Operator: '+'001'
        Sht['E1']='Date: '+dt
        
        #set header
        Sht['A2']='No'
        Sht['B2']='Source_ID'
        if self.countType[0]=='4':
            Sht['C2']='Packet_ID'
            Sht['D2']='Packet_No'
            Sht['E2']='No / Packet'
            Sht['F2']='Location'
            Sht['G2']='Remainder'
            Sht['H2']='Comments'
            No=8
        elif self.radio_state==1:
            Sht['C2']='Count'
            Sht['D2']='Comments'
            No=4
        else:
            Sht['C2']='Packet'
            Sht['D2']='No/Packet'
            Sht['E2']='Remainder'
            Sht['F2']='Comments'
            No=6
        
        i=0
        for row in range(self.itemlist):
            values_1=object.item(self.itemTv_id[row],'values')
            i +=1
            for col in range(0,No):
                c_char=get_column_letter(col+1)
                if col==0:
                    Sht[c_char+str(row+3)]=str(i)
                else:
                    Sht[c_char+str(row+3)]=values_1[col-1]
                         
        position=[200,150] #keyboard postion
        kb=keyF_keyboard(self.root,position,'File name:',d_path)
        if kb.bExit_flag:
            #use filedialog to save a file
            try:
                folderName=kb.get_Filename()
                if folderName=='':
                    folderName=d_path
                #os.chmod(folderName,stat.S_IWRITE)
                os.chmod(folderName,stat.S_IWGRP)
                fileName=folderName+'/'+kb.enVar.get()+'.xlsx'
                Ws.save(filename=fileName)
                self.bSave_flag=True
                return True
            except Exception:
                Message_dialogue(self.root,'Save file failed !','ok','Warning !') 
            finally:
                return True
        return False
     #search item in the list
    def packetID_search(self):  
        try:
            i=0
            batchNo=obj_batchInf.batch_noGet()
            for i in range(batchNo):
                values_1=self.itemTv_Lf.item(self.itemTv_id[self.lineNo[i]],'values')
                if values_1[1]==self.packet_Id:
                    tagN=self.itemTv_Lf.item(self.itemTv_id[self.lineNo[i]],'tags')
                    if tagN[0]=='bg-2':
                        return 'Duplicate barcode' 
                    #check speed
                    strSpeed=bytes(values_1[6],'utf-8')
                    iLen=len(strSpeed)
                    iFlag=True
                    j=0
                    for j in range(iLen):
                        if j==0 and strSpeed[0]==0x30:
                            iFlag=False
                            break
                        if strSpeed[0]<0x30 or strSpeed[0]>0x39:
                            iFlag=False
                            break;
                    if iFlag: 
                        self.speedBtn['text'] = f'Speed: \n{values_1[6]}'
                    else:
                        self.speedBtn['text'] = f'Speed: \n{self.speedText}'
                    #enter count process
                    No=obj_batchInf.no_perpacketGet()
                    if No[0]==0:#totalize mode
                        count.setCount()
                    else:
                        if self.tempMode==count_mode[3]:
                            count.setKernel_batch(No[i])
                        else:
                            count.setCount(1,No[i])
                           
                    self.tempMode=count_mode[0]
                    #ensure the item selected is visuable
                    self.itemTv_Lf.see(str(self.lineNo[i]))
                    self.itemTv_Lf.selection_set(str(self.lineNo[i]))
                    self.tNo=self.lineNo[i]
                    return True    
            return 'Packet ID not in list'

        except(...):
            return 'Invalid barcod'
    def sourceID_search(self): 
        #extract batch number
        batchNo=0    
        try:
            for i in range(self.start_row,self.itemlist,1):
                tagN=self.itemTv_Lf.item(self.itemTv_id[i],'tags')
                if tagN[0]=='bg-2':
                    continue
                values_1=self.itemTv_Lf.item(self.itemTv_id[i],'values')
                if values_1[0]==self.source_Id :
                    self.lineNo.append(i)
                    batchNo +=1
                    obj_batchInf.no_perbatchSet(int(values_1[2]))
                    obj_batchInf.no_perpacketSet(int(values_1[3]))
                    self.itemTv_Lf.item(self.itemTv_id[i],values=values_1,tags='bg-1')

            if batchNo ==0:
                return 'Source ID not in list'
            else:
                obj_batchInf.batch_noSet(batchNo) 
                return True
        except(...) :
            return 'Invalid barcode'    
     #get batch No
    def getBatchNo(self):
        txt=self.rsBatchLbl['text'] 
        txt_split=txt.split('\n')
        No=int(txt_split[1])
        return No
    
     #get batch_speed
    def getBatch_speed(self):
        txt=self.speedBtn['text'] 
        txt_split=txt.split('\n')
        No=int(txt_split[1])
        return No

     #set batch speed
    def setBatch_Speed(self,speedStr):
        self.h_speed=int(speedStr)
        if self.h_speed <=10:
            self.l_speed=5
        elif self.h_speed <=20:
            self.l_speed=10
        elif self.h_speed>100:
            self.l_speed=50
        else:
            self.l_speed=int((self.h_speed*2)/3)
         
     #mouse was pressed in treeview
    def mouse_Pressed(self,event):
        self.x0,self.y0=event.x,event.y
        #self.widget_name=self.root.focus_get()
        #print("Pressed:x={},y={}".format(self.x0,self.y0))
     #mouse is motion
    def mouse_Move(self,event):
         self.xm,self.ym=event.x,event.y
         #print("motion:x={},y={}".format(self.xm,self.ym))
         gap=self.y0-self.ym
         no=divmod(abs(gap),26)
         rowNo=int(no[0])
         if rowNo >1:
            if self.itemlist>11 :
                if gap < 0: #slide downward
                    nol=self.treeTop-rowNo
                    if nol>=0:
                        self.treeTop -=rowNo 
                    else:
                        self.treeTop=0
                    if self.radio_state==1:
                        self.itemTv_totalize.see(str(self.treeTop))  
                    else:
                        self.itemTv_batch.see(str(self.treeTop))  
                    
                      
                else: #slide upward
                    l=self.treeTop+11
                    nol=11-l
                    if nol>rowNo:
                        self.treeTop +=rowNo
                        if self.radio_state==1:
                            self.itemTv_totalize.see(str(l+rowNo))
                        else:
                            self.itemTv_batch.see(str(l+rowNo))
                        
                    elif nol>=0:
                        self.treeTop +=nol
                        if self.radio_state==1:
                            self.itemTv_totalize.see(str(self.itemlist-1))
                        else:
                            self.itemTv_batch.see(str(self.itemlist-1)) 
                       
     #mouse is releaseed
    def mouse_Release(self,event)    :
         self.xr,self.yr=event.x,event.y
         #print("Release:x={},y={}".format(self.xr,self.yr))
         gap=self.y0-self.yr
         no=divmod(abs(gap),26)
         rowNo=int(no[0])
         if rowNo >1:
            if self.itemlist >11:
                if gap < 0: #slide downward
                    nol=self.treeTop-rowNo
                    if nol>=0:
                        self.treeTop -=rowNo 
                    else:
                        self.treeTop=0
                    if self.radio_state==1:   
                        self.itemTv_totalize.see(str(self.treeTop))  
                    else:
                        self.itemTv_batch.see(str(self.treeTop))
                else: #slide upward
                    l=self.treeTop+11
                    nol=11-l
                    if nol>rowNo:
                        self.treeTop +=rowNo
                        if self.radio_state==1:  
                            self.itemTv_totalize.see(str(l+rowNo))
                        else:
                            self.itemTv_batch.see(str(l+rowNo))
                    elif nol>=0:
                        self.treeTop +=nol
                        if self.radio_state==1:  
                            self.itemTv_totalize.see(str(self.itemlist-1))
                        else:
                            self.itemTv_batch.see(str(self.itemlist-1))
     #Add new entry in treevie
    def additem_New(self):
        self.clear_listBtn.config(state='disabled') 
        if self.radio_BT.get() == 1: #totaliz
            #insert item into treeview for totalize
            if self.itemlist<9:
                str1='S000'+str(self.itemlist+1)
            elif self.itemlist<99:
                str1='S00'+str(self.itemlist+1)
            else:
                str1='S0'+str(self.itemlist+1)

            Source_Id=str1
            p=self.itemTv_totalize.insert("", index='end',iid=str(self.itemlist),
                                            text=str(self.itemlist+1),
                                            values=(Source_Id, " "," "),
                                            tags='bg')
                
            self.itemTv_id.append(p)
            self.itemTv_totalize.see(str(self.itemlist))
            self.itemTv_totalize.selection_set(str(self.itemlist))

            self.itemlist +=1
            self.cMode=count_mode[1]
            self.update_Treetop()
            
            self.doneBtn_flag=False

        else: #batch count
            no=obj_batchInf.batch_noGet()
            pack_No=obj_batchInf.no_perbatchGet() #retrieve packet Number
            No_perPack=obj_batchInf.no_perpacketGet()  #retrieve kernel amount per packet
            speed=obj_batchInf.speed_get()

            for x in range(no):
                if x<9:
                   Source_id='S000'+str(self.itemlist+x+1)
                elif x<99:
                   Source_id='S00'+str(self.itemlist+x+1)
                else:
                    Source_id='S0'+str(self.itemlist+x+1)

                if (self.itemlist+x) % 2==0:
                    id=self.itemTv_batch.insert("",'end',iid=str(self.itemlist+x),
                                                text=str(self.itemlist+x+1), 
                                                values=(Source_id,str(pack_No[x]),(str(No_perPack[x])+ ": "+str(speed[x]))," "," "),
                                                tags='bg'
                                                )
                else:
                    id=self.itemTv_batch.insert("",'end',iid=str(self.itemlist+x),
                                                text=str(self.itemlist+x+1), 
                                                values=(Source_id,str(pack_No[x]),(str(No_perPack[x]) +": "+str(speed[x]))," "," "),
                                                tags='bg-1'
                                                )
            
                self.itemTv_id.append(id)
            #ensure item is presented
            self.itemlist +=no
            self.update_Treetop()
           
            No=obj_batchInf.no_perpacketGet()
            count.setCount(1,No[0])
            self.speedText=str(speed[0])
            self.speedBtn.config(text=f' Speed  \n{self.speedText}')
            self.setBatch_Speed(self.speedText)
            self.dumpBtn.config(state='disabled')

     #update top number of list in treeview
    def update_Treetop(self):
        no=self.itemlist-16
        if no<=0 :
            self.treeTop=0
        else:
            self.treeTop=no 

    def radio_click(self):
         if self.radio_BT.get() ==self.radio_state:
             return
         elif self.radio_BT.get()==2:
             self.itemTv_id.clear()
             self.disable_Widgets()
             self.startBtn.config(state='disabled')
             self.countClear_click()
             #batch counter
             '''
             batchObj=batchWin(self.root,'Batch Count',
                                obj_batchInf,
                                (600,410,280,50)
                                ) 
             '''
             batchObj=batchWin(self.root,'Batch Count',
                                obj_batchInf,self.speedText,
                                (600,470,280,10)
                                ) 
             if batchObj.btn_flag : # ok button
                self.radio_state=2
                self.deltreeview_total() 
    
                self.treeview_batch()

                #set count mode
                self.cMode = count_mode[2]  #set batch count mode
                self.itemlist=0
                self.additem_New()

                #select color
                no=obj_batchInf.batch_noGet()
                self.itemTv_batch.selection_set(str(self.itemlist-no))  
                self.setTreeviewNo(no,1)
                if self.countThread_01.is_alive() ==False:
                        self.countThread_01.start() # thread has start to run

                turnTable.setSpeed_Turntab(self.speedText)
                turnTable.start_Turntab()
           
             else:  
                 self.radioBtn_t.select() 
                 self.valveBtn.config(state='normal')
                 self.startBtn.config(state='normal')
                 self.enable_Widgets()
                 self.doneBtn.config(state='disabled')
             
         else:
             #handle totalize counter
             self.itemTv_id.clear()
             self.radio_state=1
             self.deltreeview_batch()
             self.itemlist=0
             
             self.treeview_totalize()
        

    def deltreeview_total(self):
        self.itemTv_totalize.destroy()
        self.startBtn.destroy()
        self.doneBtn.destroy()
        self.valveBtn.destroy()
        self.scrobar_Treeview.pack_forget()
        self.scrobar_Treeview.destroy()
        self.itemlist=0
        
    def deltreeview_batch(self):
        self.itemTv_batch.destroy()
        self.startBtn.destroy()
        self.doneBtn.destroy()
        self.scrobar_Treeview.pack_forget()
        self.scrobar_Treeview.destroy()
        global itemTv_id
        itemTv_id.clear()   #clear
        self.itemlist=0

    def crop_click(self): 
        import cropList as aList
        crpApp=aList.cropList(self.root,self.obj_Crop)  
        self.cropText.set(crpApp.cropName)   #name
        ttK=self.obj_Crop.getcrop_Dict()
        self.cropPara_set(ttK[crpApp.cropName])
        #set counter low speed for batch count
        self.setBatch_Speed(self.speedText)
         
    def speed_click(self):
        d_keyboard=digit_keyboard(self.root,self.speedText)

        self.speedText=d_keyboard.speedVar.get()
        self.speedBtn.config(text=f' Speed  \n{self.speedText}')
        #if self.radio_BT.get()==2:
        self.setBatch_Speed(self.speedText)
        if turnTable.getState()>0:
            turnTable.start_Turntab(self.speedText)
        else:
            turnTable.setSpeed_Turntab(self.speedText)

    def countClear_click(self):
         count.clearCount()
         self.countText.set(0)
        
    def sampleReady(self):
         #pop up a alert
         logging.info("Main : Pop up a sample alert")
         self.sampleThread_01=threading.Thread(target=self.sample_Thread, name="sample",args=("Sample Ready !",))
         self.sampleThread_01.start()
         eventThread.clear()
         logging.info("Main : sample_thread has been created")

    def dumpBtn_click(self):
         #thread is at idle state
         self.cMode = count_mode[4]  #set dumpping count mode 
         
         #pop up a dumping frame
         logging.info("Main : Pop up dumpping frame")
         self.dumpThread_01=threading.Thread(target=self.dump_Thread, name="dump",args=("Dumpping...",))
         self.dumpThread_01.start()
         eventThread.clear() #set flag to false
         logging.info("Main : dump_thread has been created")
         
         #turntable start to rotating
         self.tempSpeed=self.speedText    
         turnTable.start_Turntab("200")
         self.speedBtn['text']=f' Speed  \n200'
         self.dumpBtn.config(state='disabled')
         if self.countType=='2':
            self.startBtn.config(state='disabled')
         self.disable_Widgets() 
     
    def dump_Thread(self,str_):
         
        lab=TK.Label(self.root,text=str_,
                    bg='lightblue',
                    fg='darkred',
                    font=('Times',24))
        if self.countType=='2':
            lab.place(x=750,y=635,height=60,width=250)
        else:
            lab.place(x=400,y=635,height=60,width=250)
                
        eventThread.wait()
        self.speedText=self.tempSpeed
        if self.countType=='2':
            if self.radio_BT.get() ==1:
                lab.destroy() 
                self.startBtn.config(state='normal')
                self.speedBtn['text']=f' Speed  \n{self.speedText}'
                turnTable.setSpeed_Turntab(self.speedText)
                self.enable_Widgets()
            else:
                eventThread.clear()
                lab['text']=' Discharge Sample ! '
                eventThread.wait()
                lab.destroy()
                self.startBtn.config(state='normal')
                self.back_Btn.config(state='normal')
            
        else:
            lab.destroy() 
            self.speedBtn['text']=f' Speed  \n{self.speedText}'
            turnTable.setSpeed_Turntab(self.speedText)
            self.enable_Widgets()
         
    def sample_Thread(self,str_):
         
        lab=TK.Label(self.root,text=str_,
                    bg='dark khaki',
                    fg='darkred',
                    font=('Times',25))
        lab.place(x=750,y=635,height=60,width=300)
        
        self.doneBtn.config(state='disabled')
        eventThread.wait()
        lab.destroy()
        self.doneBtn.config(state='normal')
         
    def isDone(self):
        return self.doneBtn_flag

    def startBtn_click(self):
        self.disable_Widgets()
        
        if self.radio_BT.get() ==1: #totalize
            self.doneBtn.config(state='normal')
            self.speedBtn.config(state='normal')
            if (self.buttonStart.get())=="Start":
                self.cMode = count_mode[1]  #set totalize count mode 
                self.buttonStart.set("Pause")                   
                self.additem_New()  #insert item in treeview
                count.setCount()   #totalize
                turnTable.start_Turntab()
            elif self.buttonStart.get()=="Continue":
                self.buttonStart.set("Pause") 
                turnTable.start_Turntab()
            else:
                self.buttonStart.set("Continue")
                turnTable.stop_Turntab()
        else:   #batch count
            #clear batch_inf data    
            obj_batchInf.removeData()
            self.startBtn.config(state='disabled')
            '''
            batchObj=batchWin(self.root,'Batch Count',
                                obj_batchInf,
                                (600,410,280,50)) 
            '''
            batchObj=batchWin(self.root,'Batch Count',
                                obj_batchInf,self.speedText,
                                (600,470,280,10)) 
            if batchObj.btn_flag: #ok button
                self.additem_New()

                #select color
                no=obj_batchInf.batch_noGet()
                self.itemTv_batch.selection_set((self.itemlist-no))  
                self.itemTv_batch.see(str(self.itemlist-no))
                self.setTreeviewNo(no,1)
                    
                #turntable start to rotating
                count.clearCount()
                if self.speedText =='0':
                    speed= obj_batchInf.speed_get()
                    self.speedText=str(speed[0])
                    self.speedText=str(self.speedText)
                turnTable.setSpeed_Turntab(self.speedText)
                turnTable.start_Turntab()
                self.cMode = count_mode[2]  #set batch count mode 
                self.doneBtn.config(state='normal')
            else:
                self.startBtn.config(state='normal')
                self.enable_Widgets()
                self.doneBtn.config('state=disabled')

    def doneBtnTota_click(self):
         #stop turntable  & clear counter number 
         turnTable.stop_Turntab()
         
         #thread is at idle state
         self.cMode = count_mode[0]  #set batch count mode 

         #grab values in the item
         logging.info("Main:get value %s",self.itemTv_id[self.itemlist-1])
         #set new value to Amount
         values_1=self.itemTv_totalize.item(self.itemTv_id[self.itemlist-1],"values")
         d=datetime.now()
        
         dt=d.strftime("%b %d, %H:%M") #get fromat date
         self.itemTv_totalize.item(self.itemTv_id[self.itemlist-1],
                                values=(values_1[0],str(self.countText.get()),dt),
                                tags='bg-2')
         
         #unselect 
         self.itemTv_totalize.selection_remove(self.itemTv_totalize.selection()[0])

         count.clearCount()
         self.countText.set(0)
         self.buttonStart.set("Start")
        
         self.doneBtn_flag=True

         #enable all widgits
         self.enable_Widgets()
         self.doneBtn.config(state='disabled')
         

    def doneBtnBatch_click(self):
        #stop turntable  & clear counter number 
        turnTable.stop_Turntab()
        self.speedBtn['text']=f' Speed  \n0/{self.speedText}'
        #retrieve data from treeview
        No=self.getTreeviewNo()
        
        values_1=self.itemTv_batch.item(self.itemTv_id[No],"values")
        if self.countText.get()==0:
            #get batch No
            ba_No=obj_batchInf.batch_noGet()
            No +=ba_No
            for i in range(ba_No):
                self.itemTv_batch.delete(No-i-1)
                self.itemlist -=1
            #obj_batchInf.removeData()
            self.itemTv_id.pop()    #remove is id list
        else:
            pNo1=values_1[2]+"/"+str(self.countText.get())
            #//change list state
            self.itemTv_batch.item(self.itemTv_id[No], 
                                values=(values_1[0],
                                        values_1[1],
                                        str(pNo1),
                                        'null',
                                        'partial'),
                                        tags='bg-2') 
            No +=1
            while self.itemlist >No:
                pNo1=values_1[2]+"/"+"0"
                self.itemTv_batch.item(self.itemTv_id[No], 
                                    values=(values_1[0],
                                            values_1[1],
                                            str(pNo1),
                                            'null',
                                            'null'),
                                            tags='bg-2') 
                No +=1

        #thread is at idle state
        self.cMode = count_mode[0]  #set batch count mode 
        self.doneBtn_flag=True  

        #enable widgets
        self.enable_Widgets()
        self.doneBtn.config(state='disabled')
        self.startBtn.config(state='normal')
        self.back_Btn.config(state='normal')
    
    def doneBtnFl_click(self):
        #stop turntable  & clear counter number 
        self.cMode = count_mode[0]  #set idle count mode 
        turnTable.stop_Turntab()
        #retrieve data from treeview
        No=self.getTreeviewNo()
        values_1=self.itemTv_Lf.item(self.itemTv_id[No],"values")
        packet_No=obj_batchInf.no_perbatchGet()
        if packet_No[0]==0: #handle totalize
            d=datetime.now()
            dt=d.strftime("%b %d, %H:%M") #get fromat date
            #//change list state
            self.itemTv_Lf.item(self.itemTv_id[No], 
                                values=(values_1[0],
                                        values_1[1],
                                        '0',
                                        '0',
                                        values_1[4],
                                        str(self.countText.get()),
                                        dt),
                                tags='bg-2')  
            self.T_batchNo -=1 
            self.set_countText(0)         
        else:
            batchNo=obj_batchInf.batch_noGet()
            pNo1=values_1[2]+"/"+str(self.countText.get())
            #//change list state
            self.itemTv_Lf.item(self.itemTv_id[No], 
                                values=(values_1[0],
                                        values_1[1],
                                        values_1[2],
                                        str(pNo1),
                                        values_1[4],
                                        values_1[5],
                                        'partial'),
                                        tags='bg-2')
        
            #get current batch no
            i=0
            while True:
                if No==self.lineNo[i]:
                    break
                elif i<batchNo:
                    i  +=1
                else:
                    break
            i +=1
            while i<(batchNo):
                values_1=self.itemTv_Lf.item(self.itemTv_id[self.lineNo[i]],"values")
                pNo1=values_1[2]+"/"+"0"
                self.itemTv_Lf.item(self.itemTv_id[self.lineNo[i]], 
                                values=(values_1[0],
                                        values_1[1],
                                        values_1[2],
                                        str(pNo1),
                                        values_1[4],
                                        values_1[5],
                                        'null'),
                                        tags='bg-2')
                i +=1
        
            self.doneBtn_flag=True 

            No=self.T_batchNo-batchNo
            if No==0:
                #enable widgets
                self.enable_Widgets()
                self.savefileBtn.pack(side=TK.LEFT,padx=25,pady=15,ipady=5)
                self.back_Btn.config(state='normal')
        #thread is at idle state
        lBatch_No=1
        obj_batchInf.removeData()
        self.doneBtn.config(state='disabled')
        if self.T_batchNo==0:
            self.loadfileBtn.pack(side='left',padx=40,ipady=10,ipadx=5)
            self.barcode_En.config(state='disabled')
            self.clear_listBtn.config(state='normal')
            self.savefileBtn.pack(side=TK.LEFT,padx=25,pady=15,ipady=5)
            self.barcode.set('')
        else:
            self.barcode_En.config(state='normal')
            self.barcode.set('')
            self.barcode_En.focus()
   
    def getTreeviewNo(self):
         return self.tNo

    def setTreeviewNo(self,No_batch,lBatch_No):
         self.tNo=self.itemlist-No_batch+lBatch_No-1
     
    def clearlistBtn_click(self):
        if (self.itemlist>0) and (not self.bSave_flag):
            msgbox=Message_dialogue(self.root,'Save Sample ?','yes-no','Reminder !')
            if msgbox.iflag=='OK':
                # is USB stick there?
                d_path=self.is_USB()
                if not d_path:
                    Message_dialogue(self.root,'Please insert USB stick','ok','Reminder !')           
                    return
                else:
                    if self.countType[0]=='4': #load file mode
                        self.save_file(self.itemTv_Lf,d_path)
                    elif self.radio_state==1:
                        self.save_file(self.itemTv_totalize,d_path)
                    else:
                        self.save_file(self.itemTv_batch,d_path) 
                    
        #Clear the treeview list items
        if self.countType=='2': #independent mode
            if self.radio_state ==2:
                for item in self.itemTv_batch.get_children():
                    self.itemTv_batch.delete(item)
            else :
                for item in self.itemTv_totalize.get_children():
                    self.itemTv_totalize.delete(item) 
        else:
            for item in self.itemTv_Lf.get_children():
                    self.itemTv_Lf.delete(item)
            self.loadfileBtn.pack(side='left',padx=40,ipady=10,ipadx=5)
            self.barcode_En.config(state='disabled')
            self.savefileBtn.pack_forget()
            self.start_row=0
        self.itemlist=0
        self.itemTv_id.clear()
        self.bSave_flag=False
        #hide clear button
        self.clear_listBtn.config(state='disabled')

    def is_USB(self):
        os.chdir('/media/pi')
        file_Path=os.getcwd()
        
        folder_list=os.listdir(file_Path)
        if len(folder_list)==0:
            return False
        else:
            file_Path +='/'+folder_list[0]
            return file_Path
        
    #define thread function to read counter
    def getcount_Thread(self,str1):
        global threadExit_Flag
        threadExit_Flag=True
       
        count.clearCount()
             
        oldNum=0
        newNum=0
        iChamber =0 # 0--bottom chamber is empty, 1-- bottom chamber is full
        lBatch_No =1   #local variable
        lPackage_No=1  #local variable
        iFinish = 0     #local flag, 0--not finish, 1--batch finished
        iSpeed_Change =False  # True--speed has been changed from H to Low,False--speed is at highspeed
        iRead_flag=False
        speed_stop='0'    #the speed before stop
        index_B=0   #batch sequence number for '4-y'
        No_batch=0
        No_packet=[]
        No_Kernel=[]
        speed = []

        count.countDetect()     #start to count
        
        logging.info("Thread start to read  !")

        while threadExit_Flag:
            if self.cMode==count_mode[1]:  #totalize 
                newNum = count.readCount()
                if newNum  != oldNum:
                    self.set_countText(newNum)
                    oldNum=newNum
                '''
                if envelope.getEnvelope_state() ==1:
                   if newNum>0 and turnTable.getState():
                        self.doneBtnTota_click() 
                '''
            elif self.cMode==count_mode[2] :  #batch mode
                 #Obtain batch count information
                if iRead_flag==False:
                    No_batch=obj_batchInf.batch_noGet()    #batct order
                    No_packet = obj_batchInf.no_perbatchGet()  #packet number
                    No_Kernel = obj_batchInf.no_perpacketGet() #kernel perpacket
                    speed= obj_batchInf.speed_get()
                    iRead_flag=True

                newNum = count.readCount()
                if newNum >3*No_Kernel[No_batch-1]:
                    msgbox=Message_dialogue(self.root,'Sensitivity is too high, \n First turn Sensitivity down \n then Click button: Yes or No','yes-no','alarm')
                    if msgbox.iflag=='OK':
                        #self.cMode=count_mode[0]
                        eventThread.set() #clear sample ready label
                        self.doneBtnBatch_click()
                        self.countClear_click()
                if (iChamber==0) and (valve.state_valve()==0) and iFinish==1 :
                     #go to dump
                     #unselect 
                     self.itemTv_batch.selection_remove(self.itemTv_batch.selection()[0])
                     count.setCount() 
                     self.dumpBtn.config(state='disabled')
                     self.doneBtn.config(state='disabled')  
                     self.tempMode=self.cMode
                     self.cMode=count_mode[4]

                     #prepare dumpping
                     self.tempSpeed=self.h_speed

                     eventThread.clear()
                     self.dumpThread_01=threading.Thread(target=self.dump_Thread, name="dump",args=("Dumpping...",))
                     self.dumpThread_01.start()

                     #start turn table
                     self.speed_Change('200')
                     iFinish=0
                     lPackage_No=1
                     iSpeed_Change=False
                     iRead_flag=False
                     iChamber =0 # 0--bottom chamber is empty, 1-- bottom chamber is full
                     lBatch_No =1   #local variable
                
                     speed_stop='0'   #the speed before stop
                else :
                    if newNum != oldNum:
                        self.set_countText(newNum)
                        if newNum < oldNum : #one package has been finished
                            oldNum=newNum
                            iChamber = 1    #bottom chamber
                            
                            valve.Closevalve() #close valve

                            self.sampleReady()
                           
                            #retrieve data from treeview
                            No=self.itemlist-No_batch+lBatch_No-1
                            values_1=self.itemTv_batch.item(self.itemTv_id[No],"values")
                            
                            pNo=str(lPackage_No) +"/"+ str(No_packet[lBatch_No-1])

                             #//change list state
                            self.itemTv_batch.item(self.itemTv_id[No], 
                                                    values=(values_1[0],
                                                            pNo,
                                                            values_1[2],
                                                            values_1[3],
                                                            ''))

                            #check package number
                            if lPackage_No ==No_packet[lBatch_No-1]:
                                #one batch has been completed
                                self.itemTv_batch.item(self.itemTv_id[No], 
                                                    values=(values_1[0],
                                                            pNo,
                                                            values_1[2],
                                                            values_1[3],
                                                            'complete'),
                                                            tags='bg-2')

                                if (lBatch_No) ==No_batch: # if batch has been completed
                                    turnTable.stop_Turntab()
                                    iFinish=1
                                else :
                                    count.setKernel_batch(No_Kernel[lBatch_No])
                                    lBatch_No +=1
                                    self.setTreeviewNo(No_batch,lBatch_No)
                                    self.itemTv_batch.selection_set(str(self.tNo))
                                    self.itemTv_batch.see(str(self.tNo))
                                    lPackage_No=1
                                    self.speedText=speed[lBatch_No-1]
                                    self.setBatch_Speed(self.speedText)
                                    self.speed_Change(str(self.h_speed))
                            else:
                                lPackage_No +=1
                                #back to original speed
                                self.speed_Change(str(self.h_speed))
                                iSpeed_Change=False 
                        else:
                            oldNum = newNum
                            if iChamber ==1:
                                if (newNum > No_Kernel[lBatch_No-1]/3) and (valve.state_valve()==1) and (turnTable.getState()==1) :
                                    #counter stop
                                    speed_stop=str(self.getBatch_speed())
                                    turnTable.stop_Turntab()
                                    
                                    self.speedBtn['text']=f' Speed   \n{speed_stop}'
                                if envelope.getEnvelope_state() == 1:
                                    iChamber =0
                                    eventThread.set() #clear sample
                            else : 
                                if valve.state_valve() and not envelope.getEnvelope_state() :
                                    valve.Openvalve()    
                                if iSpeed_Change==False:
                                    if (No_Kernel[lBatch_No-1] >=50 ) and (newNum > No_Kernel[lBatch_No-1]-12):
                                        self.speed_Change(str(self.l_speed))
                                        iSpeed_Change=True
                                    if (No_Kernel[lBatch_No-1] < 50 ) and (No_Kernel[lBatch_No-1] >= 20) and (newNum > No_Kernel[lBatch_No-1]-8):
                                        self.speed_Change(str(self.l_speed))
                                        iSpeed_Change=True
                                    if (No_Kernel[lBatch_No-1] < 20 ) and (newNum > No_Kernel[lBatch_No-1]-5):
                                        self.speed_Change(str(self.l_speed))
                                        iSpeed_Change=True
                    else:
                        #check envelop
                        if envelope.getEnvelope_state()==1:
                            iChamber=0
                            eventThread.set()  #clear sample alert

                        elif (iChamber==0) and (valve.state_valve()==1) and (turnTable.getState()==0) :
                            valve.Openvalve()
                            self.speed_Change(speed_stop)
                            iSpeed_Change=False 

            elif self.cMode == count_mode[3]:    #load file
                 #Obtain batch count information
                if iRead_flag==False:
                    No_batch=obj_batchInf.batch_noGet()    #batct order
                    No_packet = obj_batchInf.no_perbatchGet()  #packet number
                    No_Kernel = obj_batchInf.no_perpacketGet() #kernel perpacket
                    No=self.getTreeviewNo()   
                    index_B=self.lineNo.index(No)
                    iRead_flag=True
                newNum = count.readCount()
                if No_packet[0]==0: #totallize counter
                    
                    if newNum  != oldNum:
                        self.set_countText(newNum)
                        oldNum=newNum
                    if envelope.getEnvelope_state() ==1:
                        #if newNum>0 and turnTable.getState():
                        if newNum>0 and turnTable.getState():
                            self.doneBtnFl_click()
                else:
                    if (iChamber==0) and (valve.state_valve()==0) and iFinish==1 :
                        #go to dump
                        #unselect 
                        self.itemTv_Lf.selection_remove(self.itemTv_Lf.selection()[0])
                        
                        self.dumpBtn.config(state='disabled')
                        self.doneBtn.config(state='disabled')  
                        self.tempMode=self.cMode
                        
                        count.setCount()
                        self.cMode=count_mode[4] 
                        #prepare dumpping
                        self.tempSpeed=self.h_speed
                        self.dumpThread_01=threading.Thread(target=self.dump_Thread, name="dump",args=("Dumpping...",))
                        self.dumpThread_01.start()
                        eventThread.clear()

                        #start turn table
                        self.speed_Change('200')
                        self.T_batchNo -=No_batch
                        lBatch_No =1
                        No_batch=0
                        
                        iFinish=0
                        lPackage_No=1
                        iSpeed_Change=False
                        iRead_flag=False
                        iChamber =0 # 0--bottom chamber is empty, 1-- bottom chamber is full
                
                        speed_stop='0'    #the speed before stop
                    
                    else :
                        if newNum != oldNum:
                            self.set_countText(newNum)
                            if newNum < oldNum : #one batch has finished
                                oldNum=newNum
                                iChamber = 1    #bottom chambe
                                valve.Closevalve() #close valve
                                self.sampleReady()
                                #retrieve data from treeview
                                No=self.getTreeviewNo()
                                index_B=self.lineNo.index(No)

                                values_1=self.itemTv_Lf.item(self.itemTv_id[No],"values")
                                
                                pNo=str(lPackage_No) +"/"+ str(No_packet[index_B])

                                #//change list state
                                self.itemTv_Lf.item(self.itemTv_id[No], 
                                                    values=(values_1[0],
                                                            values_1[1],
                                                            pNo,
                                                            values_1[3],
                                                            values_1[4],
                                                            values_1[5],
                                                                ''))

                                #check package number
                                if lPackage_No ==No_packet[index_B]:
                                    self.itemTv_Lf.item(self.itemTv_id[No],
                                                        values=(values_1[0], values_1[1],pNo,values_1[3],values_1[4],values_1[5],'complete'),
                                                        tags='bg-2')
                                    if (lBatch_No) ==No_batch:
                                        turnTable.stop_Turntab()
                                        iFinish=1
                                    else :
                                        lBatch_No +=1
                                        if self.countType=='4-n':
                                            index_B +=1
                                            bStr=str(lBatch_No)+'/'+str(No_batch)
                                            self.rsBatchLbl['text']=f'BatchNo: \n{bStr}'
                                            count.setKernel_batch(No_Kernel[index_B])
                                            lPackage_No=1
                                            self.rsPacketLbl['text']=f'Packet: \n{lPackage_No}'
                                            #back to original speed
                                            self.speed_Change(str(self.h_speed))
                                            self.tNo=self.lineNo[index_B]
                                            self.itemTv_Lf.selection_set(str(self.tNo))
                                            self.itemTv_Lf.see(str(self.tNo))
                                            iSpeed_Change=False 
                                        else:
                                            #one batch has finished
                                            speed_stop=str(self.getBatch_speed())
                                            self.rsPacketLbl['text']=f'Packet: \n1'
                                            turnTable.stop_Turntab()
                                            self.tempMode=self.cMode    #package has finished       
                                else:
                                    lPackage_No +=1
                                    self.rsPacketLbl['text']=f'Packet: \n{lPackage_No}'
                                    #back to original speed
                                    self.speed_Change(str(self.h_speed))
                                    iSpeed_Change=False 
                            else:
                                oldNum = newNum
                                if iChamber ==1:
                                    if turnTable.getState()==1:
                                    #if vFeeder.getState()==1:   
                                        if (newNum > No_Kernel[index_B]/3) and (valve.state_valve()==1) :
                                            #counter stop
                                            speed_stop=str(self.getBatch_speed())
                                            turnTable.stop_Turntab()
                                            self.speedBtn['text']=' Speed   \n0'
                                    if envelope.getEnvelope_state() == 1:
                                        iChamber =0
                                        eventThread.set() #clear sample
                                else : 
                                    if valve.state_valve() and not envelope.getEnvelope_state() :
                                        valve.Openvalve()    
                                    if iSpeed_Change==False:
                                        if (No_Kernel[index_B]>=50 ) and ((newNum)> No_Kernel[0]-12):
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True
                                        if (No_Kernel[index_B] < 50 ) and (No_Kernel[0] >= 20) and ((newNum)> No_Kernel[0]-8):   
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True
                                        if (No_Kernel[index_B] < 20 ) and ((newNum)> No_Kernel[0]-5):
                                            self.speed_Change(str(self.l_speed))
                                            iSpeed_Change=True  
                        else:
                            #check envelop
                            if envelope.getEnvelope_state()==1:
                                iChamber=0
                                eventThread.set()  #clear sample alert
                                
                            elif (iChamber==0) and (valve.state_valve()==1) and (turnTable.getState()==0) :
                            #elif (iChamber==0) and (valve.state_valve()==1) and (vFeeder.getState()==0) :
                                if iFinish==0 and self.tempMode==count_mode[0]:
                                    self.speed_Change(speed_stop)
                                    iSpeed_Change=False 
                                elif self.countType=='4-y' and self.tempMode==count_mode[3]:
                                    self.itemTv_Lf.selection_clear()
                                    self.barcode_En.config(state='normal')
                                    self.barcode.set('')
                                    self.barcode_En.focus()  
                                    if iFinish==0:
                                        self.cMode=count_mode[0]
                                valve.Openvalve()                

            elif self.cMode == count_mode[4]:   #dump
                bDump=1
                iFlag=0
                envelop_flag=False   #True, havs been lift, False--not been lift

                if valve.state_valve()==1 :
                    valve.Openvalve()
                lastCount=newNum
                #set totalize count type
                countType=count.countType
                count.countType=0
                while bDump:
                    newNum = count.readCount()
                    if newNum !=oldNum :
                        oldNum=newNum
                        self.countText.set(newNum+lastCount)
                        iFlag=0
                    else :
                        if not iFlag:
                            dt1=time.time()
                            iFlag=1
                        else :
                            dt2=time.time()
                            if (dt2 - dt1)>3 : #3 secons
                                valve.Closevalve()
                                time.sleep(1)
                                valve.Openvalve()
                                turnTable.stop_Turntab()
                                
                                eventThread.set()
                                self.dumpBtn.config(state='normal')
                                bDump=0 #exit dump
                    if envelope.getEnvelope_state()==1:
                        envelop_flag=True
                '''
                if self.countType==2: #independent
                    self.enable_Widgets()
                    self.startBtn.config(state='normal')
                '''
                count.countType=countType
                d=datetime.now()
                dt=d.strftime("%b %d, %H:%M") #get fromat date
                if self.tempMode==count_mode[2] : #independent
                    #set new value to remainder
                    No=self.getTreeviewNo()
                    values_1=self.itemTv_batch.item(self.itemTv_id[No],"values")
                    self.itemTv_batch.item(self.itemTv_id[No],
                                            values=(values_1[0],
                                            values_1[1],
                                            values_1[2],
                                            self.countText.get(),
                                            dt),
                                            tags='bg-2')
                    lBatch_No=1
                    
                    global page_Back 
                    if envelop_flag==True:
                        eventThread.set()
                        time.sleep(1)
                    else:    
                        while page_Back=='':
                            if envelope.getEnvelope_state()==1:
                                eventThread.set()
                                time.sleep(1)
                                break
                    
                    self.additem_New()
                    #select color
                    no=obj_batchInf.batch_noGet()
                    self.itemTv_batch.selection_set((self.itemlist-no))  
                    self.itemTv_batch.see(str(self.itemlist-no))
                    self.setTreeviewNo(no,1)
                    self.speed_Change(self.speedText)
                    self.countClear_click()
                    self.cMode=count_mode[2]
                    self.startBtn.config(state='disable')
                    self.doneBtn.config(state='normal')

                elif self.tempMode==count_mode[3]:
                    #set new value to remainder
                    no=self.getTreeviewNo()
                    values_1=self.itemTv_Lf.item(self.itemTv_id[no],"values")
                    self.itemTv_Lf.item(self.itemTv_id[no],
                                            values=(values_1[0],
                                                    values_1[1],
                                                    values_1[2],
                                                    
                                                    values_1[3],
                                                    values_1[4],
                                                    str(self.countText.get()),
                                                    dt),
                                                    tags='bg-2')
                    lBatch_No=1
                    self.doneBtn.config(state='disabled')
                    obj_batchInf.removeData()
                    if self.T_batchNo==0:
                        self.loadfileBtn.pack(side='left',padx=40,ipady=10,ipadx=5)
                        self.barcode_En.config(state='disabled')
                        self.clear_listBtn.config(state='normal')
                        self.savefileBtn.pack(side=TK.LEFT,padx=25,pady=15,ipady=5)
                        self.barcode.set('')
                    else:
                        self.barcode_En.config(state='normal')
                        self.barcode.set('')
                        self.barcode_En.focus()
                    self.countClear_click()
                    #count.clearCount()
                    #self.set_countText(0)
                    self.rsBatchLbl['text']='BatchNo:\n1'
                    self.rsPacketLbl['text']='Packet:\n1'
                    self.cMode=count_mode[0]
                else:    
                    self.cMode=count_mode[0]
                    self.speedBtn['text']=f' Speed  \n{self.speedText}'
                eventThread.set()
                oldNum=0
                newNum=0
                self.tempMode=count_mode[0]
            else:
                newNum= count.readCount()
                if newNum !=oldNum:
                    oldNum=newNum
                    self.set_countText(newNum)
                if self.countType=='4-n':
                    lBatch_No=1
                lPackage_No=1
                iFinish=0
                iChamber=0
                iSpeed_Change=False
                iRead_flag=False
                speed_stop='0'

        logging.info("Thread exit counting !")
     
    def valveBtn_click(self):
         if (self.buttonValve.get())=="Close valve" :
             self.buttonValve.set("Open Valve")
             valve.Closevalve()
         else:
            self.buttonValve.set("Close valve")
            valve.Openvalve()

    def back_click(self):
        global page_Back
        turnTable.stop_Turntab()
        if (self.itemlist>0) and (not self.bSave_flag):
            msgbox=Message_dialogue(self.root,'Save Sample ?','yes-no','Reminder !')
            if msgbox.iflag=='OK':
                d_path=self.is_USB()
                if not d_path:
                    Message_dialogue(self.root,'Please insert USB stick','ok','Reminder !')          
                    return 
                else:
                    if self.countType[0]=='4':
                            self.save_file(self.itemTv_Lf,d_path)
                    elif self.radio_state==1:
                        self.save_file(self.itemTv_totalize,d_path)
                    else:
                        self.save_file(self.itemTv_batch,d_path)              

        page_Back= "back"
        # exit thread of getcount_thread
        global threadExit_Flag
        threadExit_Flag = False
        self.countThread_01.join() #wait thread exit
        self.clearObje()
        self.powerThread.join()
        self.poweroffThread_save.join()
        page_Back= ''
        self.root.destroy()
    #change speed
    def speed_Change(self,intVal):#intVal dats type is string 
        strVal=intVal
        strSpeed=f'Speed: \n{strVal}'
        self.speedBtn['text']=strSpeed
        turnTable.start_Turntab(strVal)

    #get current batch No
    def get_cBatch_No(self):
        cBatch=self.rsBatchLbl['text']
        x=cBatch.find('/')
        if x==-1:
            return 0
        else:
            tt=cBatch[9:x]
            return int(tt)
    #change count display
    def set_countText(self,val) :
         self.countText.set(val)

    def cropPara_set(self,K):
        self.speedText=K[1]
        self.speedBtn.config(text=f' Speed  \n{self.speedText}')     #speed
        self.sensitivityLb1.config(text=f' Sensitvity \n{K[0]}')    #sensitivity
        t= int(K[2])/1000
        valve.setDelay_time(t)          #set delay time of valve close
        self.horiLb1.config(text=f'  HoriGate  \n{K[3]}')        #horigate
        self.vertiLb1.config(text=f' VertiGate  \n{K[4]}')          #herGate 
        turnTable.setSpeed_Turntab(int(K[1]))

    #disable widgets
    def disable_Widgets(self) :
         self.back_Btn.config(state='disabled')
         self.clear_listBtn.config(state='disabled')
         self.dumpBtn.config(state='disabled')
         self.speedBtn.config(state='disabled')
         self.doneBtn.config(state='disabled')

         if self.countType=='2':
            self.radioBtn_t.config(state='disabled')
            self.radioBtn_b.config(state='disabled')
            self.cropBtn.config(state='disabled')
   
    #enable widgets
    def enable_Widgets(self):
        self.back_Btn.config(state='normal')
        self.clear_listBtn.config(state='normal')
        self.dumpBtn.config(state='normal')
        self.speedBtn.config(state='normal')
        
        if self.countType=='2':
            self.radioBtn_t.config(state='normal')
            self.radioBtn_b.config(state='normal')
            self.cropBtn.config(state='normal')
            if self.radio_state==1:
                self.doneBtn.config(state='normal')
           

    def clearObje(self):
         count.cleanDetect()
         envelope.cleanDetect()
         obj_batchInf.removeData()
    
       
"""
define counter state
"""
class mainPage_030(frlsWin):
    def __init__(self):
        super().__init__()  
        from PIL import ImageTk, Image
        self.speed=0

        #load arrow image
        img_up=ImageTk.PhotoImage(Image.open("/home/pi/esc2/up arrow.gif"))
        img_down=ImageTk.PhotoImage(Image.open("/home/pi/esc2/down arrow.gif"))
      
        #change title of window
        self.label['text']="Counter State"

        #define font
        self.myFont=Font(family="Times New Roman",size=20)
        self.ttk_Style=TTK.Style(self.root)
        
        #create two frames for layout, frm1 for left, frm2 for right
        self.frm1=TK.Frame(self.root,bg="SlateBlue3",
                        bd=1,
                        relief='ridge'
                        )

        self.frm2=TK.Frame(self.root,bg="SlateBlue3",
                        bd=1,
                        relief='ridge'
                        )
        
        self.frm1.pack(side='left',pady=20,padx=20)
       
        #create a frame for speed check
        speedstateFrm = TK.Frame(self.frm1,bg='SlateBlue3')
        
        speedBtn = TK.Button(speedstateFrm,text="1.Feed speed: ",
                            bg="gray",
                            font=self.myFont,
                            command=self.speedBtn_click
                            )
        speedBtn.grid(row=1,column=1,ipadx=8,ipady=5)

        panel_up=TK.Label(speedstateFrm,image=img_up)
        panel_up.bind("<Button-1>",self.up_click)

        panel_up.grid(row=1,column=2,ipadx=5,ipady=5,padx=20)

        self.speedLbl = TK.Label(speedstateFrm,text='0',
                                 bg="gray",relief=TK.RIDGE,
                                 width=5,
                                 font=self.myFont
                                 )
        self.speedLbl.grid(row=1,column=3,ipadx=8,ipady=5,padx=20)

        panel_down=TK.Label(speedstateFrm,image=img_down)
        panel_down.bind("<Button-1>",self.down_click)

        panel_down.grid(row=1,column=4,ipadx=5,ipady=5,padx=20)

        self.speedstopBtn =TK.Button(speedstateFrm,text="Stop",
                                     bg="gray",relief=TK.RAISED,
                                     font=self.myFont,
                                     command=self.speedstopBtn_click
                                     )
        self.speedstopBtn.grid(row=1,column=5,ipadx=8,ipady=5,padx=20)

        #set position of the speed state
        speedstateFrm.pack(fill='x',pady=20,padx=50)

        #create a frame for valve time delay
        valveTimeFrm = TK.Frame(self.frm1,bg='SlateBlue3')
        self.valvedelayBtn = TK.Button(valveTimeFrm,text='2.Valve Delay Time:',
                                bg="gray",
                                font=self.myFont,
                                command=self.delay_Time
                                )
        self.valvedelayBtn.grid(row=1,column=1,ipadx=8,ipady=5)

        t=int(1000*valve.getDelay_time())
        self.vDelayLbl =TK.Label(valveTimeFrm,text=str(t),
                                bg='gray',width=5,
                                relief=TK.RIDGE,
                                font=self.myFont
                                )
        self.vDelayLbl.grid(row=1,column=2,padx=20,ipady=8)

        timeLbl=TK.Label(valveTimeFrm,text='ms',
                                bg='gray',
                                font=self.myFont
                                )
        timeLbl.grid(row=1,column=3,padx=20,ipadx=5,ipady=5)
        valveTimeFrm.pack(fill='x',pady=20,padx=50)

        #create a frame for valve state
        valvestateFrm = TK.Frame(self.frm1,bg='SlateBlue3')

       
        self.valveBtn = TK.Button(valvestateFrm,text='3.Main Valve',
                                  bg="gray",relief=TK.RAISED,
                                  font=self.myFont,
                                  command=self.valveBtn_click
                                  )
        self.valveBtn.grid(row=1,column=1,ipadx=5,ipady=5,sticky='w')

        self.valveStatus=TK.StringVar()
        if valve.state_valve() :#closed
            self.valveStatus.set('Closed')
        else:
            self.valveStatus.set('Opened')

        valvestateLbl = TK.Label(valvestateFrm,textvariable=self.valveStatus,
                                    bg="gray",
                                    relief=TK.RIDGE,
                                    font=self.myFont
                                    )
        valvestateLbl.grid(row=1,column=2,ipadx=8,ipady=5,padx=20)

        dischargeLbl1 = TK.Label(valvestateFrm,text='4. Discharge gate \n \n Lift the discharge bar\n for discharge\n',
                                      bg='gray',
                                      fg='red',
                                      font=self.myFont
                                      )
        dischargeLbl1.grid(row=2,column=1,ipadx=15,ipady=6,pady=50)
        
        self.envelopeStatus=TK.StringVar()
        if envelope.getEnvelope_state() :#envelope present, opened
            self.envelopeStatus.set('Opened')
        else:
            self.envelopeStatus.set('Closed')
        self.dischargebarstateLbl = TK.Label(valvestateFrm,textvariable=self.envelopeStatus,
                                             bg="gray",
                                             relief=TK.RIDGE,
                                             font=self.myFont
                                             )
        self.dischargebarstateLbl.grid(row=2,column=2,ipadx=5,ipady=5,pady=50)
        #set position of the valve state
        valvestateFrm.pack(fill='x',pady=30,padx=50)                                     
        
        #obtain IP address:
        import subprocess
        Ip_add=str(subprocess.check_output(['hostname','-I'])).split(' ')[0].replace("b'","")
        IpaddrLab=TK.Label(self.frm1,text='IP:'+Ip_add+' '+'Port:502',
                            bg='gray',
                            relief=TK.RIDGE,
                            font=self.myFont)
        IpaddrLab.pack()
        
        #create a check button
        checkFrm=TK.Frame(self.frm2,bg='SlateBlue3')
        self.CheckBn=TK.StringVar()
        self.CheckBn.set('hi')
        global rs232_carousel
        rs232_carousel='1'
        self.ttk_Style.configure('TCheckbutton',font=('Times New Roman',30),indicatorsize=50)
        carousel_Ch=TTK.Checkbutton(checkFrm,text='Carousel',
                                   variable=self.CheckBn,
                                   onvalue='hi',
                                   offvalue='bye',
                                   style="TCheckbutton",
                                   command=self.Check_Btn)
        
        
        carousel_Ch.pack(fill='x',padx=30,pady=10)
        checkFrm.pack(fill='x',pady=10,padx=50)
        
        #create a frame for sensitivity
        senFrm=TK.Frame(self.frm2,bg='SlateBlue3')
        
        titleLabel=TK.Label(senFrm,text='--Sensitivity test--',
                            bg='gray',
                            font=('Times New Roman',16)
                            )
        titleLabel.pack(fill='x',pady=5)
        
        inforLabel=TK.Label(senFrm,text='Use hand to drop seed\n at each channel',
                                    bg='gray',
                                    fg='red',
                                    font=self.myFont)
        inforLabel.pack(fill='x')
        senFrm.pack(fill='x',pady=20,padx=50)
        
        #create a frame for count
        countFrm=TK.Frame(self.frm2,bg='SlateBlue3')
        countLbl=TK.Label(countFrm,text="Count :",
                               bg="gray",
                               font=self.myFont
                               )
        countLbl.grid(row=1,column=1,ipadx=5,ipady=5)

        self.countnumLbl= TK.Label(countFrm,text="  0",
                                    bg="gray",
                                    font=self.myFont
                                    )
        self.countnumLbl.grid(row=1,column=2,ipady=5, padx=20) 

        self.countclearBtn = TK.Button(countFrm,text="Clear",
                                       bg="gray",relief=TK.RAISED,
                                       font=self.myFont,
                                       command=self.countclearBtn_click
                                       )

        self.countclearBtn.grid(row=1,column=3,ipadx=5,ipady=5,padx=20)
        countFrm.pack(fill='both',padx=50,pady=80)
        self.frm2.pack(side='left',pady=20,padx=20)

        #create a frame for back button
        self.backFrm = TK.Frame(self.root,bg='light slate blue')
        
        self.backBtn = TK.Button(self.backFrm,text="Home",
                                  bg="gray",relief=TK.RAISED,
                                  font=self.myFont,
                                  command=self.backBtn_click)
                                  
        self.backBtn.pack(side=TK.LEFT,ipadx=5,ipady=5)

        #set position of back button
        self.backFrm.place(x=920,y=640)
        
        #create a thread to detect discharge bar & sensitivity
        self.thread_Detect()

        self.root.mainloop()
   
    def speedBtn_click(self):
        self.speed=int(self.speedLbl['text'])
        d_keyboard=digit_keyboard(self.frm1,self.speed)

        self.speed=int(d_keyboard.speedVar.get())
    
        turnTable.start_Turntab(self.speed)
        self.speedLbl['text']=str(self.speed)

    def up_click(self,event):
        if self.speed <200:
            self.speed +=1
            turnTable.start_Turntab(self.speed)
            self.speedLbl['text']=str(self.speed)

    def down_click(self,event):
        if self.speed>0:
            self.speed -=1
            turnTable.start_Turntab(self.speed)
            self.speedLbl['text']=str(self.speed)
  
    def speedstopBtn_click(self):
        self.speed=int(turnTable.getSpeed_Turntab())
        turnTable.stop_Turntab()
        self.speedLbl['text']='0'

    def delay_Time(self):
        #appear keyboard
        dTime=int(self.vDelayLbl['text'])
        bt=digit_keyboard(self.frm1,dTime)
        dTime=bt.speedVar.get()
        self.vDelayLbl['text']=dTime

        t=int(dTime)/1000
        valve.setDelay_time(t)
        
    def countclearBtn_click(self):
        count.clearCount()
        self.countnumLbl['text']='0'
        self.seedNo_new=0
        self.seedNo_old=0
    def Check_Btn(self):
        global rs232_carousel 
        if (self.CheckBn.get())=='hi':
            rs232_carousel='1'
        else:
            rs232_carousel='0'
        
    def valveBtn_click(self):
        if valve.state_valve():#closed
            valve.Openvalve()
            self.valveStatus.set('Opened')
        else:
            valve.Closevalve()
            self.valveStatus.set('Closed')

    def thread_Detect(self):
        self.thread_No=threading.Thread(target=self.detect_DC,
                                    name='detector',
                                    args='0'
                                    )
        self.thread_No.setDaemon(True)
        logging.info("Main:Thread detector has been created")
        self.thread_No.start()

    def detect_DC(self,strVal):
        global threadExit_Flag
        threadExit_Flag=True

        envelope.envelop_Detect()
        # detect sensitivity for each channel 
        count.setCount()
        count.clearCount()
        count.countDetect()
        self.seedNo_new=0
        self.seedNo_old=0
        while threadExit_Flag:
            self.seedNo_new=count.readCount()
            if self.seedNo_new >self.seedNo_old:
                self.seedNo_old=self.seedNo_new
                self.countnumLbl['text']=str(self.seedNo_old)
            #get envenlop state
            if envelope.getEnvelope_state():
                if self.envelopeStatus.get()=='Closed':
                    self.envelopeStatus.set('Opened')   
            else:
                if self.envelopeStatus.get()=='Opened':
                    self.envelopeStatus.set('Closed')

            time.sleep(0.1)  
        print('Thread: detect_DC has exited !')

    def backBtn_click(self):
        global page_Back
        page_Back= "back"

        global threadExit_Flag
        threadExit_Flag = False

        #waiting thread exit
        self.thread_No.join()

        count.cleanDetect()
        envelope.cleanDetect()
        self.powerThread.join()
        page_Back=''
        self.root.destroy()
    
if __name__ =="__main__":
    while True:     
        obj0=mainPage() 
        if obj0.page_Index !='':        
            if obj0.page_Index=="1": #RS232 mode
                obj1=mainPage_010('rs232')
                del obj1
            elif obj0.page_Index=="2":   #independent modethanksn1e6b4
                obj2=mainPage_020('2')
                del obj2
            elif obj0.page_Index=="3":   #counter state
                obj3= mainPage_030()
                del obj3
            elif obj0.page_Index[0]=='4':   #load file
                obj4=mainPage_020(obj0.page_Index)
                del obj4
            elif obj0.page_Index=="5":
                obj5=mainPage_010('rs485tcp')
                del obj5
            elif obj0.page_Index=="6":
                obj6=mainPage_010('rs485rtu')
                del obj6
        else:
            break
    print("Program has exit !")


    
    

