"""
-------------------------------------------------------------------------------------------
|June 4, 2021| Daniel T.| this module will implememt a control class 
|Sep 30, 2022| Daniel T.| add new class to test vibrating Feeder
|Oct 04, 2023| Daniel T.| add new class: cCount_N
|March 5, 2025, remove self.over_d =0 in method setKernel_batch()
--------------------------------------------------------------------------------------------
| class cValve:
|   1. OpenValve()  ,no return
|   2. CloseValve() ,no return
|   3. state_valve() ,return valve state, 1--close, 0--open
|-------------------------------------------------------------------------------------------
|class  cEnvelop:
|   1.  getEnvelop_state()  , return envelop_state  1---envelope present, 0---not present
|   2.  envelop_Detect() , test risingand falling edge
|   3.  envelop_Callback()     , check whether a envelope present or not
|-------------------------------------------------------------------------------------------
|class cTurntable:
|   1.  getSpeed_Turntab()  , return current speed
|   2.  start_Turntab(str)  , str=-1, -1 means to use default setting
|   3.  stop_Turntab()      , stop turntable
|   4.  setSpeed_Turntab(str),  
|   the duty of pulse is 50
|-------------------------------------------------------------------------------------------
|class cCount:
|   1.  countDetect()   ,set an interrupt for comming pulse
|   2.  count_Callback()    ,handle income pusle
|   3.  readCount()
|   4.  setCount(val1,val2)     , val1--0:totalize, 1--batch
|   5.  clearCount()
|--------------------------------------------------------------------------------------------
|class cCount_N:   **Oct 06, 2023 | Daniel T.
|   member varible:
|       Over_D--ripple carry output
|       lCount--Count number
|       countPackage--No of package
|        sountType--countType
|   member function:
|       __init__ 
|       __del__
|       readCount(), return a count
|       clearCount()
|       setCount(val1,val2)
|       countDetect(); detect ripple carry pin
|       countCallback()
|-------------------------------------------------------------------------------------------
|class cPowerM
|   1.  isPowerOn(),    True--Power on , False--Power off
|   2.  power_detect(), detect a edge down
|   3.  power_callback(), handle power off
|   4.  cleanDetect(),   clear function
|   5.  TurnOff19(), turnoff main circuit board, August 9, 2024
|   4.  cP_Flag , TRUE--POWER is turn off
|-------------------------------------------------------------------------------------------
|class testPulse:   imitate incomming external pulse
|   1.  testStart()
|   2.  testStop()
|-------------------------------------------------------------------------------------------
|version1.0   | Jun 18, 2021
|------------------------------------------------------------------------------------------
|             | Jan 19, 2023 | modified code , make more accuracy
|             | Oct 06,2023 | modify count class cCount_n
-----------------------------------------------------------------------------------------
"""
import time
import logging
from tkinter.constants import FALSE, TRUE
from xml.sax.handler import property_declaration_handler
import RPi.GPIO as GPIO


"""
set globale variable for GPIO
"""
valve_open_pin = 17  # GPIO17
valve_close_pin = 22  # GPIO22
motor_pulse_pin = 12  # PWM0, GPI012
envelope_state_pin = 27  # GPIO27
count_pin = 5  # GPIO05 , not used oct 04, 2023
sda_count = 2  # GPIO2
scl_count = 3  # GPIO3
i2c_Address = 0x60  # pic address

power_detect_pin = 13  # GPIO13
power_on_pin = 19  # GPIO19
N555_dis_pin = 26  # GPIO26
power_detect_pin = 13  # GPIO13
'''
define date input port
'''
port_A=9        #bit0
port_B=10       #bit1
port_C=11       #bit2
port_D=23       #bit3

port_RCO=8    #RCO 
port_CLR=25   #Clear to zero


#for rs485 control
rs485_RE=6  #GPIO6 low level,enable to read data
rs485_DE=16 #GPIO16, SHDN pin. high level,enable chip work


testcount_pin = 13     # PWM1 GPIO13

#format1 = "%(asctime)s: %(message)s"
#logging.basicConfig(format=format1, level=logging.INFO, datefmt="%H:%M:%S")

# use BCM mode
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

'''
define Power control class
'''
class cPower:
    def __init__(self, pin1=power_detect_pin):
        self.power_detect_pin = pin1
        self.power_on_pin = power_on_pin
        self.cP_Flag = FALSE  # Power on
        
        GPIO.setup(self.power_detect_pin,GPIO.IN,pull_up_down=GPIO.PUD_UP)
        GPIO.setup(self.power_on_pin,GPIO.OUT,initial=GPIO.HIGH)
        
    def __del__(self) :
        self.cleanDetect()
           
    def isPowerOn(self):
        return self.cP_Flag

    def power_detect(self):
        GPIO.add_event_detect(self.power_detect_pin,GPIO.FALLING,callback=self.power_callback,bouncetime=5)

    def power_callback(self, power_detect_pin):
        if not GPIO.input(self.power_detect_pin):
            time.sleep(1)
            if not GPIO.input(self.power_detect_pin):
                self.cP_Flag=TRUE   
                
    def cleanDetect(self):
        GPIO.remove_event_detect(self.power_detect_pin)
        
    def TurnOff19(self):#output low to  turn off maintain system
        GPIO.output(self.power_on_pin,GPIO.LOW)


"""
define valve class
class member: valve_state. 0--valve opened, 1--valve closed, 2--indefined
"""
class cValve:
    def __init__(self, val1=valve_open_pin, val2=valve_close_pin):  # contructor of class
        self.vClose_pin = val1
        self.vOpen_pin = val2
        self.valve_state = 2  # indefined
        self.dTime=0.150#default setting

        # initial a connection
        #self.vPi = GPIO.pi()
        GPIO.setup(self.vClose_pin,GPIO.OUT,initial=GPIO.LOW)
        GPIO.setup(self.vOpen_pin,GPIO.OUT,initial=GPIO.LOW)
        
        self.initai_Valve()

    def __del__(self):  # deconstructor of class
        GPIO.cleanup(self.vClose_pin)
        GPIO.cleanup(self.vOpen_pin)
        time.sleep(1)
        GPIO.setup(self.vClose_pin,GPIO.OUT,initial=GPIO.LOW)
        GPIO.setup(self.vOpen_pin,GPIO.OUT,initial=GPIO.LOW)
        
        #self.vPi.stop()

    # initial valve state
    def initai_Valve(self):
        self.Closevalve()
        time.sleep(1.5)
        self.Openvalve()

    # open valve
    def Openvalve(self):
        # ensure close valve in state of low volt     
        if not GPIO.input(self.vClose_pin):
            GPIO.output(self.vClose_pin,GPIO.LOW)
            time.sleep(0.02)

        GPIO.output(self.vOpen_pin,GPIO.HIGH)
        time.sleep(0.05) #hold open 50 ms
        self.valve_state=0 #open state
        GPIO.output(self.vOpen_pin,GPIO.LOW)

    # close valve
    def Closevalve(self):
        # ensure open valve at state of low volt
        if not GPIO.input(self.vOpen_pin):
            GPIO.output(self.vOpen_pin,GPIO.LOW)
            time.sleep(0.02)

        time.sleep(self.dTime)
        GPIO.output(self.vClose_pin,GPIO.HIGH)
        time.sleep(0.05)  #hold close 50ms
        self.valve_state=1; #close state
        GPIO.output(self.vClose_pin,GPIO.LOW)

        
    # valve state
    def state_valve(self):
        return self.valve_state
    
    #set delay time
    def setDelay_time(self,val):
        self.dTime=val     #val: delay time s
    
    #get delay time
    def getDelay_time(self):
        return self.dTime


"""
    envelope_state 0--envelope dose'nt present, 
                   1-- envelope present
"""


class cEnvenlop:
    def __init__(self, val=envelope_state_pin):
        self.envelope_state_pin = envelope_state_pin
        
        GPIO.setup(self.envelope_state_pin,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
        self.envelop_state = 0  # envelope dose not present

    def __del__(self):
        GPIO.remove_event_detect(self.envelope_state_pin)
        GPIO.cleanup(self.envelope_state_pin)

    def cleanDetect(self):
        GPIO.remove_event_detect(self.envelope_state_pin)

    def getEnvelope_state(self):
        
        if GPIO.input(self.envelope_state_pin):
            self.envelop_state=1    #envelope present           
        else :
            self.envelop_state=0    #envelop dosen't present

        return self.envelop_state

    def envelop_Callback(self, evenlope_state_pin):
        
        if GPIO.input(self.envelope_state_pin):
            logging.info("envelope did present!")
            self.envelop_state=1
        else:
            logging.info("envelope did not present!")
            self.envelop_state=0
        
    def envelop_Detect(self):
        
        GPIO.add_event_detect(self.envelope_state_pin,GPIO.RISING, callback=self.envelop_Callback,bouncetime=5)
        

"""
define a class to control turn table
the range of speed: 0~200, 0--standstill, 200---the most speed
member: iSpeed
member: iState  0--standstill, 1--run
method:
1--stop(), 
2--setspeed(str), 
3--getspeed()   return a value of the speed
4--getState()   return run or standstill
"""


class cTurntab:
    def __init__(self, val=motor_pulse_pin):  # constructor
        self.Speed = 50  # default setting is 50
        self.iState = 0  # standstill
        self.tPulse_Pin = val
        
        #GPIO.setup(self.tPulse_Pin,GPIO.OUT,initial=GPIO.LOW)
        
        GPIO.setup(self.tPulse_Pin,GPIO.OUT,initial=GPIO.HIGH)
        self.tT=GPIO.PWM(self.tPulse_Pin,500)   #set default frequeny 125hz --50
       
    def __del__(self):  # distructor
        GPIO.cleanup(self.tPulse_Pin)
        del self.tT
        #cPi.set_PWM_dutycycle(self.tPulse_Pin, 0)  # stop pwm
        #cPi.stop()

    # stop turntap
    def stop_Turntab(self):
        self.tT.stop()
        time.sleep(0.050)
        #cPi.set_PWM_dutycycle(self.tPulse_Pin, 0)  # stop pwm
        GPIO.output(self.tPulse_Pin,GPIO.HIGH)
        self.iState = 0

    # start turntab
    def start_Turntab(self, Str_="-1"):  # -1 using defualt setting or last time setting
        val = int(Str_)
        if Str_ == '-1':
            self.setSpeed_Turntab(self.Speed)
            self.tT.start(50) # duty cycle=50
            #cPi.set_PWM_dutycycle(self.tPulse_Pin, 128)  # dutycycle 50%
        elif val == 0:
            self.Speed = 0
            self.stop_Turntab()
        else:
            self.setSpeed_Turntab(Str_)
            self.tT.start(50) #duty cycle=50
            #cPi.set_PWM_dutycycle(self.tPulse_Pin, 128)  # dutycycle 50%

        if val == 0:
            self.iState = 0
        else:
            self.iState = 1

    # set speed
    def setSpeed_Turntab(self, Str_):
        val = int(Str_)
        if val > 0 and val<=50:
            fT=(val*500/200)+1
            self.tT.ChangeFrequency(fT)
        else :
            fT=val*500/200+int((val/50)*5)
            
            self.tT.ChangeFrequency(fT)
        self.Speed = val

    def getSpeed_Turntab(self):
        return str(self.Speed)

    def getState(self):
        return self.iState

"""
define a class to control vibrating Feeder
the frequency: 30~100Hz, Duty: 1%-99%
0--standstill, 200---the most speed
member: iSpeed
member: iState  0--standstill, 1--run
method:
1--stop_VFeededr(), 
2--start_VFeeder(fe=0,du=0). fe--frquency, Duty cycle
2--setspeed_VFeeder(fe), 
3--getspeed()   return a value of the speed
4--getState()   return run or standstill
"""
class cVFeeder:
    def __init__(self, val=motor_pulse_pin):  # constructor
        self.Frequency=73
        # default setting is 73Hz
        self.Duty = 50 # default duty is 50%
        self.iState = 0  # standstill
        self.tPulse_Pin = val
        
        GPIO.setup(self.tPulse_Pin,GPIO.OUT,initial=GPIO.LOW)
        self.tT=GPIO.PWM(self.tPulse_Pin,self.Frequency)   #set default frequeny 70Hz
       
    def __del__(self):  # distructor
        GPIO.cleanup(self.tPulse_Pin)
        del self.tT

    # stop turntap
    def stop_VFeeder(self):
        self.tT.stop()
        self.iState = 0

    # start turntab
    def start_VFeeder(self, dut='-1'):  # -1 using defualt setting or last time setting
        du=int(dut)
        if du ==-1:
            self.tT.ChangeFrequency(self.Frequency)
            self.tT.start(self.Duty) # duty cycle=48
        elif du==0:
            self.Duty=0
            self.stop_VFeeder()
        else :
            self.setSpeed_VFeeder(du)
            self.tT.start(self.Duty) # duty cycle=50
        
        if du==0:
            self.iState=0
        else:
            self.iState=1

    # set speed
    def setSpeed_VFeeder(self, fe):
        fre=float(fe)
        
        if fre<20:
            self.Frequency=69
            self.Duty=fre+20
        elif fre>=20 and fre<30:
            self.Frequency=71
            self.Duty=fre+20
        elif fre<=50 and fre>=30:
            self.Frequency=73
            self.Duty=fre
        elif fre>50 and fre<=100:
            self.Frequency=74
            self.Duty=0.15*fre+40.0
        elif fre>100 and fre<200:
            self.Frequency=75
            self.Duty=0.15*fre+40.0
        
        else:
            self.Frequency=75
            self.Duty=65.0
        #self.stop_VFeeder()    
        self.tT.ChangeFrequency(self.Frequency)
    #change duty
    def changeDuty_VFeeder(self,du):
        due=int(du)
        self.Duty=due
        self.tT.start(self.Duty)

    def getSpeed_VFeeder(self):
        return [self.Frequency,self.Duty]

    def getState(self):
        return self.iState
"""
define: Raspberry Pi Write options:
        1.lCount--count number
        2. countType : 0--totalize count, 1--batch count
        3. countPakage: Number perpackage
        3
"""
class cCount:
    def __init__(self, val=count_pin):
        self.count_pin = val
        self.lCount = 0
        self.oldCount = 0
        self.countPackage = 0
        self.countType = 0  # default is totalize 0
        GPIO.setup(self.count_pin,GPIO.IN,pull_up_down=GPIO.PUD_UP)
        
    def __del__(self):
        GPIO.remove_event_detect(self.count_pin)
        GPIO.cleanup(self.count_pin)
        self.cleanDetect()

    def cleanDetect(self):
        GPIO.remove_event_detect(self.count_pin)
        
    def setCount(self, countType=0, valPackage=0):
        self.countType = countType  # totalize or batch
        self.countPackage = valPackage  # per-package number for batch
        self.lCount = 0
        self.oldCount = 0
        
    def setKernel_batch(self,intVal):
        self.countPackage = intVal

    # callback of interrupt
    def count_Callback(self, count_pin):
        self.lCount += 1
    
    # define read count4
    def readCount(self):
        if self.countType == 1:
            if self.lCount > self.countPackage:
                self.lCount = 0
                
        self.oldCount = self.lCount
        return self.oldCount

    # interrup setting
    def countDetect(self):
        GPIO.add_event_detect(self.count_pin,GPIO.FALLING, callback=self.count_Callback)
            
    def clearCount(self):
        self.lCount = 0
        self.oldCount = 0

'''
define a class to read number from a counter chip sn74ls161an
date:Oct 06 2023   | Daniel T.
    member varible:
        Over_D--ripple carry output
        lCount--Count number
        countPackage--No of package
        sountType--countType
    member function:
        __init__ 
        __del__
        readCount(), return a count
        clearCount()
        setCount()
        countDetect(); detect ripple carry pin
        countCallback()
'''
class cCount_N:
    def __init__(self,val=port_RCO):
        
        self.over_D=0
        self.lCount = 0
        self.lCount_old=0
        self.lCount_1_old=0
        self.countPackage = 0
        self.countType = 0  # default is totalize 0
        self.lCount_previous=0 # for multiple counting
        
        self.RCO=val
        
        GPIO.setup(port_A,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
        GPIO.setup(port_B,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
        GPIO.setup(port_C,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
        GPIO.setup(port_D,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
       
        #clear counter
        GPIO.setup(port_CLR,GPIO.OUT,initial=GPIO.HIGH)
        
        #set ripple carry output pin
        GPIO.setup(self.RCO,GPIO.IN,pull_up_down=GPIO.PUD_DOWN)
    
    def __del__(self):
        #clear counter
        GPIO.output(port_CLR,GPIO.LOW)
        #clean Counter
        self.clearCount()
        
        #clear detect
        self.cleanDetect()
        #cleanup pin set
        GPIO.cleanup([port_A,port_B,port_C,port_D,port_CLR])
        
    def cleanDetect(self):
        GPIO.remove_event_detect(port_RCO)  
          
    def readCount(self):
        A_3=GPIO.input(port_D)
        A_2=GPIO.input(port_C) 
        A_1=GPIO.input(port_B) 
        A_0=GPIO.input(port_A)      
          
        lCount_1=A_0+A_1*2+A_2*4+A_3*8
    
        self.lCount=lCount_1 + self.over_D*16
        '''
        if self.lCount-self.lCount_old>=16:
            if lCount_1>=self.lCount_1_old:
                print('self.over_D=%d',self.over_D) 
                self.over_D -=1
                self.lCount=lCount_1 + self.over_D*16
        '''
        if self.lCount > self.lCount_old:
            self.lCount_old=self.lCount
            self.lCount_1_old=lCount_1
        else:
            self.lCount=self.lCount_old
            
        if self.countType == 1:
            if self.lCount+self.lCount_previous>=self.countPackage:
                self.over_D=0
                #clear counter
                GPIO.output(port_CLR,GPIO.LOW)
                time.sleep(0.01) #delay 10ms
                GPIO.output(port_CLR,GPIO.HIGH)
                self.lCount=self.lCount+self.lCount_previous-self.countPackage
                self.lCount_old=self.lCount
                self.lCount_previous=self.lCount
                
        return self.lCount
    
    def clearCount(self):
        #clear counter
        GPIO.output(port_CLR,GPIO.LOW)
        time.sleep(0.02) #delay 20ms
        GPIO.output(port_CLR,GPIO.HIGH)
        self.lCount = 0
        self.lCount_old=0
        self.lCount_1_old=0
        self.over_D=0
        self.lCount_previous=0
        
    def setCount(self, countType=0, valPackage=0):
        self.clearCount()
        self.countType = countType  # totalize or batch
        self.countPackage = valPackage  # per-package number for batch
    
    def setKernel_batch(self,intVal):
        #self.over_D=0
        self.countPackage = intVal
      
    # interrup setting
    def countDetect(self):  
        GPIO.add_event_detect(self.RCO, GPIO.FALLING, callback=self.count_Callback)
                              #bouncetime=50)  #bouncetime =50millisecond 
    
    # callback of interrupt
    def count_Callback(self, port_RCO):
        self.over_D += 1 
        #print('self.over_D=%d',self.over_D) 
        #logging.info('self.over_d= %d',self.over_D)

               
'''
create a rs485_RTU class
1. read_ePin: low-enable to read, high-disable
2. chip_ePin: high-enable chip wor, low-shutdownchip
'''
class rs485set:
    def __init__(self,pin1=rs485_RE,pin2=rs485_DE):
        self.read_ePin=pin1
        self.chip_ePin=pin2
        #disable rs485
        GPIO.setup(self.read_ePin,GPIO.OUT,initial=GPIO.HIGH)
        GPIO.setup(self.chip_ePin,GPIO.OUT,initial=GPIO.LOW)
    
    def read485(self):#enable to read rs 485
        GPIO.output(self.chip_ePin,GPIO.HIGH)   #enable chip normal work
        time.sleep(0.5)
        GPIO.output(self.read_ePin,GPIO.LOW)    #enable receiver 

    def send485(self):
        GPIO.output(self.read_ePin,GPIO.HIGH)   #disable read
        time.sleep(0.5)
        GPIO.output(self.chip_ePin,GPIO.HIGH)   #enable chip 

    def dis485(self):#disable rs485
        GPIO.output(self.read_ePin,GPIO.HIGH)   #disable read
        time.sleep(0.5)
        GPIO.output(self.chip_ePin,GPIO.LOW)    #disable chip

# define testing class to imitate pulse for test purpose
class testPulse:
    def __init__(self, val=testcount_pin):
        # for test
        self.pulsePin = val
        GPIO.setup(self.pulsePin, GPIO.OUT, initial=GPIO.LOW)
        # set default frequeny 125hz --50
        self.testC = GPIO.PWM(self.pulsePin, 50)

    def __del__(self):
        #GPIO.cleanup(self.pulsePin)
        del self.testC

    # imitate external count
    def test_start(self):
        self.testC.ChangeFrequency(50)
        self.testC.start(50)  # duty is 50

    def test_stop(self):
        self.testC.stop()


if __name__ == '__main__':
    # define logging format
    format1 = "%(asctime)s: %(message)s"
    logging.basicConfig(format=format1, level=logging.INFO, datefmt="%H:%M:%S")

    #valve = cValve()  # initial valve class
    #turnTable = cTurntab()  # initial turntab class
    count = cCount_N()  # initial count
    #envelope = cEnvenlop()  # initial envelope class

    # test=testPulse()

    # following code for testing valve, PWM, counting

    # Define a function for

    count.countDetect()

    while True:
        print("Please select:\n")
        choice =input("1--Valve\n2--TurnTable\n3--envelope\n4--count\n5--Vfeeder\n6--exit")
       
        choice = choice
        if choice == '1':
            '''
            st = valve.state_valve()
            if st == 1:  # close state
                st1 = input("Please input O to open valve, or q to quit text :\n")
                if st1 == 'o' or 'O':
                    valve.Openvalve()
            else:  # open statte
                st1 = input("Please input C to close valve, or q to quit text :\n")
                if st1 == 'c' or 'C':
                    valve.Closevalve()
            '''
            print('has been shielded')
            break
        elif choice == '2':
            '''
            turnTable.setSpeed_Turntab(50)  # set turntable speed
            turnTable.start_Turntab()
            time.sleep(10)
            turnTable.stop_Turntab()
            '''
            print('has been shielded')
            break
        elif choice == '3':
            logging.info("Test envelope sensor")
            '''
            envelope.envelop_Detect()
            while True:
                if envelope.getEnvelope_state():
                    logging.info("Envelop presents !")
                    break
            logging.info("exit test envelope !")
            '''
            print('has been shielded')
            break
        elif choice == '4':
            
            logging.info("Creat a thread for count")

            count.setCount(0)

            t = 0
            oldval = 0
            logging.info("Start reading--")
            #turnTable.setSpeed_Turntab(50)  # set turntable speed
            #turnTable.start_Turntab()
            while True:
                val = count.readCount()
               
                if val != oldval:
                    t += 1
                    logging.info(
                        "No.{0}--Counter:{1}".format(str(t), str(val)))
                    oldval = val
                if val > 30000:
                    # test.test_stop()
                    logging.info("Value:{}. Stop reading---".format(val))
                    #turnTable.stop_Turntab()
                    break
                #time.sleep(0.001)   
            print('has been shielded')
            break
            logging.info("Counting finished !")
        elif choice =='5':
            VFeeder =cVFeeder()
            print('VFeeder has been selected\n')
            print('\n')
            print('has been shielded')
            while True:
                select_ch =input("1--Start\n2--Stop\n3--change frequency\n4--change Duty\n5--change frequency and duty\n6--exit\n")
                if select_ch=='1':
                    VFeeder.start_VFeeder()
                    print('Frequency=',VFeeder.Speed)
                    print('Duty=',VFeeder.Duty)
                elif select_ch=='2':
                    VFeeder.stop_VFeeder()
                elif select_ch=='3':
                    VFeeder.stop_VFeeder()
                    cFre=input("please input frequency. range: 30~100 ")
                    fre=int(cFre)
                    VFeeder.start_VFeeder(0,fre)
                    print('Frequency=',VFeeder.Speed)
                    print('Duty=',VFeeder.Duty)
                elif select_ch=='4':
                    cDuty=input("please input Duty cycle. range: 1~99 ")
                    duty=int(cDuty)
                    VFeeder.changeDuty_VFeeder(duty,0)
                    print('Frequency=',VFeeder.Speed)
                    print('Duty=',VFeeder.Duty)
                elif select_ch=='5':
                    VFeeder.stop_VFeeder()
                    cFre=input("please input frequency. range: 30~100 ")
                    fre=int(cFre)
                    cDuty=input("please input Duty cycle. range: 1~99 ")
                    duty=int(cDuty)
                    VFeeder.start_VFeeder(duty,fre)
                    print('Frequency=',VFeeder.Speed)
                    print('Duty=',VFeeder.Duty)
                else:
                    del VFeeder
                    print('Vibrating feeder object has been deleted !')
                    break
            break
        else:
            logging.info("Exit program !")
            break

